unit ConMap;
{
	GearHead: Arena, a roguelike mecha CRPG
	Copyright (C) 2005 Joseph Hewitt

	This library is free software; you can redistribute it and/or modify it
	under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation; either version 2.1 of the License, or (at
	your option) any later version.

	The full text of the LGPL can be found in license.txt.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
	General Public License for more details. 

	You should have received a copy of the GNU Lesser General Public License
	along with this library; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
}

interface

uses
{$IF DEFINED(ASCII_SDL)}
  {$IFDEF SDL2}
	SDL2,
  {$ELSE SDL2}
	SDL,
  {$ENDIF SDL2}
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt,
{$ELSE}
	crt,
{$ENDIF}
{$IFDEF PATCH_GH}
	gears_base,
{$ENDIF PATCH_GH}
	gears,locale;

const
	{ DISPLAYSHOT CONSTANTS }
	SHOT_Hit = 0;
	SHOT_Dodge = 1;
	SHOT_Parry = 2;

	TerrGfx: Array [1..NumTerr] of Char = (
		'.', '=', '#', '=', '%',
		'.', '.', '^', '^', '^',
		'.', '#', '#', '.', '#',
		'.', '.', '#', '+', '.',
		'=', '-', '#', '#', '.',
		'_', '#', '.', '#', '.',
		'#', '$', '#', '#', '#',
		'$', '$', '$', '.', '=',
		'-', '#'
	);
{$IFDEF ASCII_SDL}
	TerrColor: Array [1..NumTerr] of TSDL_Color = (
		( r:  0; g:192; b:  0 ),	{ Green, }
		( r:127; g:255; b:127 ),	{ LightGreen, }
		( r:127; g:255; b:127 ),	{ LightGreen, }
		( r:127; g:127; b:255 ),	{ LightBlue, }
		( r:127; g:127; b:127 ),	{ DarkGray, }

		( r:192; g:192; b:192 ),	{ LightGray, }
		( r:  0; g:192; b:192 ),	{ Cyan, }
		( r:127; g:127; b:127 ),	{ DarkGray, }
		( r:192; g:192; b:192 ),	{ LightGray, }
		( r:255; g:255; b:255 ),	{ White, }

		( r:127; g:127; b:127 ),	{ DarkGray, }
		( r:192; g:192; b:192 ),	{ LightGray, }
		( r:192; g:192; b:192 ),	{ LightGray, }
		( r:127; g:127; b:127 ),	{ DarkGray, }
		( r:127; g:127; b:127 ),	{ DarkGray, }

		( r:192; g:192; b:192 ),	{ LightGray, }
		( r:192; g:192; b:  0 ),	{ Brown, }
		( r:192; g:  0; b:  0 ),	{ Red, }
		( r:  0; g:192; b:192 ),	{ Cyan, }
		( r:127; g:127; b:255 ),	{ LightBlue, }

		( r:  0; g:  0; b:192 ),	{ Blue, }
		( r:  0; g:  0; b:192 ),	{ Blue, }
		( r:255; g:255; b:255 ),	{ White, }
		( r:255; g:255; b:127 ),	{ Yellow, }
		( r:192; g:  0; b:  0 ),	{ Red, }

		( r:192; g:192; b:  0 ),	{ Brown, }
		( r:192; g:192; b:  0 ),	{ Brown, }
		( r:  0; g:  0; b:192 ),	{ Blue, }
		( r:  0; g:192; b:192 ),	{ Cyan, }
		( r:192; g:192; b:192 ),	{ LightGray, }

		( r:192; g:192; b:  0 ),	{ Brown, }
		( r:127; g:255; b:127 ),	{ LightGreen, }
		( r:192; g:  0; b:192 ),	{ Magenta, }
		( r:127; g:255; b:255 ),	{ LightCyan, }
		( r:  0; g:  0; b:192 ),	{ Blue, }

		( r:255; g:  0; b:255 ),	{ LightMagenta, }
		( r:192; g:192; b:  0 ),	{ Brown, }
		( r:  0; g:192; b:192 ),	{ Cyan, }
		( r:255; g:255; b:127 ),	{ Yellow, }
		( r:192; g:192; b:  0 ),	{ Brown, }

		( r:192; g:  0; b:192 ),	{ Magenta, }
		( r:255; g:  0; b:255 )		{ LightMagenta }
	);
{$ELSE ASCII_SDL}
	TerrColor: Array [1..NumTerr] of Byte = (
		Green, LightGreen, LightGreen, LightBlue, DarkGray,
		LightGray, Cyan, DarkGray, LightGray, White,
		DarkGray, LightGray, LightGray, DarkGray, DarkGray,
		LightGray, Brown, Red, Cyan, LightBlue,
		Blue, Blue, White, Yellow, Red,
		Brown, Brown, Blue, Cyan, LightGray,
		Brown,LightGreen,Magenta,LightCyan,Blue,
		LightMagenta,Brown,Cyan,Yellow,Brown,
		Magenta,LightMagenta
	);
{$ENDIF ASCII_SDL}


{$IFDEF ASCII_SDL}
Function TeamColor( GB: GameBoardPtr; G: GearPtr ): TSDL_Color;
{$ELSE ASCII_SDL}
Function TeamColor( GB: GameBoardPtr; G: GearPtr ): Byte;
{$ENDIF ASCII_SDL}
Function ScreenX( X: Integer ): Integer;
Function ScreenY( Y: Integer ): Integer;
Function OnTheScreen( X , Y: Integer ): Boolean;
Function OnTheScreen( Mek: GearPtr ): Boolean;
Function NeedsRecentering( X,Y: Integer ): Boolean;

procedure DisplayMap( gb: GameBoardPtr );

Procedure RedrawTile( gb: GameBoardPtr; X,Y: Integer );
procedure RedrawTile( gb: GameBoardPtr; Mek: GearPtr );

procedure IndicateTile( GB: GameBoardPtr; X , Y: Integer );
procedure IndicateTile( GB: GameBoardPtr; Mek: GearPtr );

Procedure RevealMek( GB: GameBoardPtr; Mek,Spotter: GearPtr );
Procedure VisionCheck( GB: GameBoardPtr; Mek: GearPtr );

Procedure DeployMek( GB: GameBoardPtr; Mek: GearPtr; PutOnMap: Boolean );
Procedure DeployMek( GB: GameBoardPtr; Mek,Pilot: GearPtr; Team: Integer );
Function LocateMekByUID( GB: GameBoardPtr; UID: Integer ): GearPtr;

Function TimeString( ComTime: LongInt ): String;
Procedure UpdateCombatDisplay( GB: GameBoardPtr );
Procedure GFCombatDisplay( GB: GameBoardPtr );

Procedure FocusOnMek( GB: GameBoardPtr; Mek: GearPtr );

Function DisplayEffectAnimations( GB: GameBoardPtr; N: Integer ): Boolean;

Procedure DisplayConsoleHistory( GB: GameBoardPtr );

Procedure ProcessMovement( GB: GameBoardPtr; Mek: GearPtr );

Procedure WriteCampaign( Camp: CampaignPtr; var F: Text );
Function ReadCampaign( var F: Text ): CampaignPtr;

Procedure BeginTurn( GB: GameBoardPtr; M: GearPtr );


implementation

uses
{$IFDEF DEBUG}
	sysutils,
	errmsg,
{$ENDIF DEBUG}
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl,
{$ENDIF}
{$IFDEF PATCH_I18N}
	i18nmsg,
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_GH}
	ui4gh,
	ability,action,damage,effects,gearutil,ghprop,menugear,movement,
	texutil,congfx,context
{$ELSE PATCH_GH}
     ability,action,damage,effects,gearutil,ghprop,menugear,movement,
     texutil,ui4gh,congfx,context
{$ENDIF PATCH_GH}
{$IFDEF PATCH_HEART}
	,qrl
{$ENDIF PATCH_HEART}
	;

const
	OriginX: Integer = 1;	{ These constants tell what tile is being }
	OriginY: Integer = 1;	{ displayed in the upper left corner of the }
				{ map area. }



	FROZEN_MAP_CONTINUE = 1;
	FROZEN_MAP_SENTINEL = -1;


{$IFDEF ASCII_SDL}
Function TeamColor( GB: GameBoardPtr; G: GearPtr ): TSDL_Color;
{$ELSE ASCII_SDL}
Function TeamColor( GB: GameBoardPtr; G: GearPtr ): Byte;
{$ENDIF ASCII_SDL}
	{ Select a good color based upon the team this }
	{ gear belongs to. }
var
{$IFDEF ASCII_SDL}
	T: LongInt;
	color: TSDL_Color;
{$ELSE ASCII_SDL}
	T,color: LongInt;
{$ENDIF ASCII_SDL}
begin
{$IFDEF PATCH_GH}
	if (NIL = G) or (G^.G <= GG_DisposeGear) or (NIL = GB) then begin
{$ELSE PATCH_GH}
	if ( G = Nil ) or ( GB = Nil ) then begin
{$ENDIF PATCH_GH}
		{ No gear provided - Neutral Gray. }
		color := NeutralGrey;

	end else if not GearOperational( G ) then begin
		{ Nonfunctioning gear. Color = DarkGrey. }
		color := DarkGray;

	end else begin
		T := NAttValue( G^.NA , NAG_Location , NAS_Team );

		if T = NAV_DefPlayerTeam then begin
			{ Player team. Color = Blue. }
			color := PlayerBlue;

		end else if AreEnemies( GB , NAV_DefPlayerTeam , T ) then begin
			{ Enemy team. Color = Red. }
			color := EnemyRed;

		end else if AreAllies( GB , NAV_DefPlayerTeam , T ) then begin
			{ Ally team. Color = Purple. }
			color := AllyPurple;

		end else begin
			{ Neutral team. Color = Brown. }
			color := NeutralBrown;

		end;
	end;
	TeamColor := Color;
end;


Function ScreenX( X: Integer ): Integer;
	{ Return the screen coordinates of map column X. }
begin
	ScreenX := X - OriginX + ScreenZone[ZONE_Map,1];
end;

Function ScreenY( Y: Integer ): Integer;
	{ Return the screen coordinates of map row Y. }
begin
	ScreenY := Y - OriginY + ScreenZone[ZONE_Map,1];
end;

Function OnTheScreen( X , Y: Integer ): Boolean;
	{ This function returns TRUE if the specified point is visible }
	{ on screen, FALSE if it isn't. }
var
	SX,SY: Integer;		{ Find Screen X and Screen Y and see if it's in the map area. }
begin
	SX := ScreenX( X );
	SY := ScreenY( Y );
	if ( SX >= ScreenZone[ZONE_Map,1] ) and ( SX <= ScreenZone[ZONE_Map,3] ) and ( SY >= ScreenZone[ZONE_Map,2] ) and ( SY <= ScreenZone[ZONE_Map,4] ) then begin
		OnTheScreen := True;
	end else begin
		OnTheScreen := False;
	end;
end;

Function OnTheScreen( Mek: GearPtr ): Boolean;
	{ Check to see whether or not the specified mek is visible on screen. }
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}
	OnTheScreen := OnTheScreen( NAttValue( Mek^.NA , NAG_Location , NAS_X ) , NAttValue( Mek^.NA , NAG_Location , NAS_Y ) );
end;

Function MapDisplayHeight: Integer;
	{ Return the height of the map display, in tiles. }
begin
	MapDisplayHeight := ScreenZone[ZONE_Map,4] - ScreenZone[ZONE_Map,2] + 1;
end;

Function MapDisplayWidth: Integer;
	{ Return the width of the map display, in tiles. }
begin
	MapDisplayWidth := ScreenZone[ZONE_Map,3] - ScreenZone[ZONE_Map,1] + 1;
end;

Procedure RecenterDisplay( X , Y: Integer );
	{ Change the display so that point X,Y will be on screen. }
begin
	OriginX := X - (MapDisplayWidth div 2);
	OriginY := Y - (MapDisplayHeight div 2);

	if OriginX < 1 then
		OriginX := 1
	else if OriginX > (XMax - MapDisplayWidth) then
		OriginX := XMax - MapDisplayWidth + 1;

	if OriginY < 1 then
		OriginY := 1
	else if OriginY > (YMax - MapDisplayHeight) then
		OriginY := YMax - MapDisplayHeight + 1;

end;

Function NeedsRecentering( X,Y: Integer ): Boolean;
	{ Check point X,Y to see whether or not the screen needs to be }
	{ centered upon it. }
var
	RC: Boolean;
begin
	{ Start by assuming FALSE, then checking the various cases }
	{ to see if a recentering is called for. }
	RC := False;

	{The screen will be recentered if ScreenX or ScreenY are within 3 }
	{ squares of the edge of the display area, and that said edge is not}
	{the edge of the map.}
	if ( ScreenX( X ) <= 3 ) and ( OriginX > 1) then
		RC := True
	else if ( ScreenX( X ) >= (MapDisplayWidth - 3)) and ( OriginX < (XMax - MapDisplayWidth + 1)) then
		RC := True;

	if ( ScreenY( Y ) <= 3 ) and ( OriginY > 1) then
		RC := True
	else if ( ScreenY( Y ) >= (MapDisplayHeight - 3)) and (OriginY < (YMax - MapDisplayHeight + 1)) then
		RC := True;

	NeedsRecentering := RC;
end;


{$IFDEF ASCII_SDL}
Procedure DrawMapImage( Gfx: Char; X,Y: Integer; C: TSDL_Color );
{$ELSE ASCII_SDL}
Procedure DrawMapImage( Gfx: Char; X,Y: Integer; C: Byte );
{$ENDIF ASCII_SDL}
	{ Draw the specified image at the specified map coordinates. }
begin
	if not OnTheScreen( X , Y ) then exit;
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.GotoXY( ScreenX( X ) , ScreenY( Y ) );
	ascii_sdl.TextColor( C );
	ascii_sdl.TextBackground( StdBlack );
	ascii_sdl.Write( Gfx );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.GotoXY( ScreenX( X ) , ScreenY( Y ) );
	w32crt.TextColor( C );
	w32crt.TextBackground( StdBlack );
	w32crt.Write( Gfx );
{$ELSE}
	GotoXY( ScreenX( X ) , ScreenY( Y ) );
	TextColor( C );
	TextBackground( StdBlack );
	Write( Gfx );
{$ENDIF}
end;

{$IFDEF ASCII_SDL}
Procedure DrawRvsImage( Gfx: Char; X,Y: Integer; C: TSDL_Color );
{$ELSE ASCII_SDL}
Procedure DrawRvsImage( Gfx: Char; X,Y: Integer; C: Byte );
{$ENDIF ASCII_SDL}
	{ Draw the specified image at the specified map coordinates. }
begin
	if not OnTheScreen( X , Y ) then exit;
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.GotoXY( ScreenX( X ) , ScreenY( Y ) );
	ascii_sdl.TextColor( StdBlack );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.GotoXY( ScreenX( X ) , ScreenY( Y ) );
	w32crt.TextColor( StdBlack );
{$ELSE}
	GotoXY( ScreenX( X ) , ScreenY( Y ) );
	TextColor( StdBlack );
{$ENDIF}
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.TextBackground( C );
	ascii_sdl.Write( Gfx );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	if ( C = Black ) or ( C = DarkGray ) then C := Blue;
	w32crt.TextBackground( C );
	w32crt.Write( Gfx );
{$ELSE}
	if ( C = Black ) or ( C = DarkGray ) then C := Blue;
	TextBackground( C );
	Write( Gfx );
{$ENDIF}
end;

procedure DrawMekX( GB: GameBoardPtr; Mek: GearPtr; Hilight: Boolean );
	{ Draw a mecha indicator at map location X , Y with }
	{ direction D. Got all that? }
var
	GFX: Char;
	roguechar: String;	{ ASCII display character. }
{$IFDEF ASCII_SDL}
	Color: TSDL_Color;
{$ELSE ASCII_SDL}
	Color: Byte;
{$ENDIF ASCII_SDL}
	X,Y: Integer;
begin
	{ Error check- make sure we have a valid mek. }
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ELSE PATCH_GH}
	if Mek = Nil then Exit;
{$ENDIF PATCH_GH}
	if not OnTheScreen( Mek ) then Exit;

	{ Extract the position information. }
	X := NAttValue( Mek^.NA , NAG_Location , NAS_X );
	Y := NAttValue( Mek^.NA , NAG_Location , NAS_Y );

	{ Error check - make sure the mecha is on the map. }
	if not OnTheMap(X,Y) then exit;

	{ Make sure the mek is visible to the player... }
{$IFDEF DEBUG}
	if (MekVisible( gb , Mek ) or DEBUG_CanSeeAll) then begin
{$ELSE DEBUG}
	if MekVisible( gb , Mek ) then begin
{$ENDIF DEBUG}
		roguechar := SAttValue( Mek^.SA , 'ROGUECHAR' );

		{ If the mek is destroyed, draw wreckage instead of the regular image. }
		if ( Mek^.G <> GG_MetaTerrain ) and not IsMasterGear( Mek ) then begin
			Gfx := '-';
			Color := NeutralGrey;
		end else if Destroyed( Mek ) then begin
			Gfx := '%';
			Color := NeutralGrey;
		end else begin
			{ Pick an appropriate character for the mek. }
			{ If the mek is at a smaller scale than the map, }
			{ pick a smaller character. }
			if Mek^.Scale < ( GB^.Scale - 3 ) then begin
				Gfx := ',';
			end else if Mek^.Scale < GB^.Scale then begin
				Gfx := 'o';
			end else if roguechar <> '' then begin
				Gfx := roguechar[1];
			end else if ( NAttValue( Mek^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam ) and ( NumActiveMasters( GB , NAV_DefPlayerTeam ) = 1 ) then begin
				Gfx := '@';
			end else begin
{$IFDEF PATCH_I18N}
				Gfx := InitialGearName( Mek );
{$ELSE PATCH_I18N}
				Gfx := GearName( Mek )[1];
{$ENDIF PATCH_I18N}
			end;
			if Mek^.G = GG_MetaTerrain then begin
				case Mek^.S of
					GS_MetaCloud:	if SAttValue( Mek^.SA , 'EFFECT' ) = '' then Color := LightGray
							else Color := LightGreen;
					GS_MetaFire:	Case Random( 3 ) of
								0,1:	Color := LightRed;
								2:	Color := Yellow;
							end;
				else Color := Yellow;
				end;

			end else begin
				Color := TeamColor( GB , Mek );
			end;
		end;

		if HiLight then DrawMapImage( Gfx , X , Y , StdWhite )
		else DrawMapImage( Gfx , X , Y , Color );
	end;
end;

procedure DrawMek( gb: GameBoardPtr; Mek: GearPtr );
	{ Draw a mecha indicator at map location X , Y with }
	{ direction D. Got all that? }
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	DrawMekX( gb , Mek , False );
end;

procedure PlotTerrainX( gb: GameBoardPtr; X,Y: Integer; Hilight: Boolean );
	{ Display the terrain in location X,Y. }
	{ This is the X-tended version of the function, with more features. }
begin
	{ Error check - make sure the requested tile is on the map, }
	{ and is also on the screen. }
	if not OnTheMap( X , Y ) then exit;
	if not OnTheScreen( X , Y ) then exit;

	{ If this tile is marked as visible better show it. }
	{ Otherwise draw a blank spot. }
{$IFDEF DEBUG}
	if (gb^.Map[ X , Y ].Visible or DEBUG_CanSeeAll) then begin
{$ELSE DEBUG}
	if gb^.Map[ X , Y ].Visible then begin
{$ENDIF DEBUG}
		if Hilight then DrawRvsImage( TerrGfx[ gb^.map[X,Y].terr ] , X , Y , TerrColor[ gb^.map[X,Y].terr ] )
		else DrawMapImage( TerrGfx[ gb^.map[X,Y].terr ] , X , Y , TerrColor[ gb^.map[X,Y].terr ] );
	end else begin
		if Hilight then DrawRvsImage( '?' , X , Y , Blue )
		else DrawMapImage( ' ' , X , Y , Black );
	end;
end;

procedure PlotTerrain( gb: GameBoardPtr; X,Y: Integer );
	{ Display the terrain in location X,Y. }
begin
	PlotTerrainX( gb , X , Y , False );
end;

procedure DisplayMap( gb: GameBoardPtr );
	{ Actually display the map. }
var
	X,Y: Integer;
	N,S,W,E: Boolean;
	M: GearPtr;
begin
	{ Begin by clearing the screen & doing the border. }
	{ Figure out in which directions the map extends. }
	N := OriginY > 1;
	W := OriginX > 1;
	E := OriginX <= ( XMax - MapDisplayWidth );
	S := OriginY <= ( YMax - MapDisplayHeight );
	DrawMapBorder( N , E , S , W );

	{ Display all terrain. }
	for X := 0 to ( MapDisplayWidth - 1 ) do begin
		for Y := 0 to ( MapDisplayHeight - 1 ) do begin
			PlotTerrain( gb , X + OriginX , Y + OriginY );
		end;
	end;

	{ Display all stationary objects... }
	{ i.e. root level nonmaster gears. }
	{ Items only get displayed if the tile they're in is already visible. }
	M := gb^.Meks;
	while M <> Nil do begin
{$IFDEF PATCH_GH}
	    if (GG_DisposeGear < M^.G) then begin
{$ENDIF PATCH_GH}
		if not IsMasterGear(M) then begin
			X := NAttValue( M^.NA , NAG_Location , NAS_X );
			Y := NAttValue( M^.NA , NAG_Location , NAS_Y );
  {$IFDEF DEBUG}
			if OnTheMap( X , Y ) and (GB^.Map[X,Y].Visible or DEBUG_CanSeeAll) then begin
  {$ELSE DEBUG}
			if OnTheMap( X , Y ) and GB^.Map[X,Y].Visible then begin
  {$ENDIF DEBUG}
				DrawMek( GB , M );
			end;
		end;
{$IFDEF PATCH_GH}
	    end;
{$ENDIF PATCH_GH}
		M := M^.Next;
	end;

	{ Display the mecha, i.e. the master gears. }
	M := gb^.Meks;
	while M <> Nil do begin
{$IFDEF PATCH_GH}
	    if (GG_DisposeGear < M^.G) then begin
{$ENDIF PATCH_GH}
		if IsMasterGear(M) then begin
			DrawMek( GB , M );
		end;
{$IFDEF PATCH_GH}
	    end;
{$ENDIF PATCH_GH}
		M := M^.Next;
	end;

	{ While we're here, redo the shadow map. }
	UpdateShadowMap( GB );
end;

Procedure RedrawTileX( gb: GameBoardPtr; X,Y: Integer; Hilight: Boolean );
	{ Redraw the tile at X,Y including all mecha and }
	{ stuff that might be occupying this spot. }
var
	M: GearPtr;	{ Mecha, maybe... }
begin
	if OnTheMap( X , Y ) and OnTheScreen( X , Y ) then begin
		{ Seek a master gear at this spot, since that'll appear }
		{ on top of everything else. }
		M := FindVisibleBlockerAtSpot( GB , X , Y );

		{ If there's a visible gear here, plot that. }
{$IFDEF PATCH_GH}
		if (NIL <> M) and (GG_DisposeGear < M^.G) then begin
{$ELSE PATCH_GH}
		if M <> Nil then begin
{$ENDIF PATCH_GH}
			DrawMekX( GB , M , Hilight );

		end else begin
			{ No master here, check for an item. }
			M := FindVisibleItemAtSpot( GB , X , Y );

{$IFDEF PATCH_GH}
			if (NIL <> M) and (GG_DisposeGear < M^.G) and GB^.Map[X,Y].Visible then begin
{$ELSE PATCH_GH}
			if ( M <> Nil ) and GB^.Map[X,Y].Visible then begin
{$ENDIF PATCH_GH}
				{ Draw the item. }
				DrawMekX( GB , M , Hilight );
			end else begin
				{ If not, just display the terrain. }
				PlotTerrainX( gb , X , Y , Hilight );
			end;
		end;
	end;
end;

Procedure RedrawTile( gb: GameBoardPtr; X,Y: Integer );
	{ Just call the above procedure with a Hilight value of FASLE. }
begin
	RedrawTileX( gb , X , Y , False );
end;

procedure RedrawTile( gb: GameBoardPtr; Mek: GearPtr );
	{ Display the tile containing the listed gear. }
	{ If the tile is not currently visible, the map will be recentered. }
var
	Team: Integer;
	P: Point;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	team := NAttValue( Mek^.NA , NAG_Location , NAS_Team );
	P := GearCurrentLocation( Mek );
	if ( Team = NAV_DefPlayerTeam ) and NeedsRecentering( P.X , P.Y ) then begin
		RecenterDisplay( P.X , P.Y );
		DisplayMap( gb );
	end else begin
		RedrawTile( gb , P.X , P.Y );
	end;
end;

procedure IndicateTile( GB: GameBoardPtr; X , Y: Integer );
	{Indicate the desired tile with a rectangle of the requested color. }
begin
	if not OnTheMap( X , Y ) then exit;
	if NeedsRecentering( X , Y ) then begin
		RecenterDisplay( X , Y );
		DisplayMap( gb );
	end;
	RedrawTileX( GB , X , Y , True );
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.GotoXY( ScreenX( X ), ScreenY( Y ) );
{$ELSE}
	GotoXY( ScreenX( X ), ScreenY( Y ) );
{$ENDIF}
end;

procedure IndicateTile( GB: GameBoardPtr; Mek: GearPtr );
	{ Indicate the tile containing the listed gear. }
var
	team: Integer;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	team := NAttValue( Mek^.NA , NAG_Location , NAS_Team );
	if MekVisible( GB , Mek ) and ( OnTheScreen( Mek ) or ( Team = NAV_DefPlayerTeam ) ) then IndicateTile( GB , NAttValue( Mek^.NA , NAG_Location , NAS_X ) , NAttValue( Mek^.NA , NAG_Location , NAS_Y ) );
end;

Procedure RevealMek( GB: GameBoardPtr; Mek,Spotter: GearPtr );
	{ This mek has been spotted. Light it up. }
var
	team: Integer;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
	if (NIL = Spotter) or (Spotter^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	team := NAttValue( Spotter^.NA , NAG_Location , NAS_Team );
	SetNAtt( Mek^.NA , NAG_Visibility , Team , NAV_Spotted );
	RedrawTile( gb , Mek );
end;

Procedure CheckVisibleArea( GB: GameBoardPtr; Mek: GearPtr );
	{ Expand the visual area around this model. }
var
	P: Point;
	X,Y,MZ,R,Obs: Integer;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	P := GearCurrentLocation( Mek );
	R := MappingRange( Mek , GB^.Scale );
	MZ := MekAltitude( GB , Mek );

	{ Look through every tile within range. If it's on the map and }
	{ not yet revealed, do a check to see if it should be. }
	for X := ( P.X - R ) to ( P.X + R ) do begin
		for Y := ( P.Y - R ) to ( P.Y + R ) do begin
			if OnTheMap( X , Y ) and not GB^.Map[X,Y].Visible then begin
				{ This tile will be revealed if Range + Obscurement }
				{ is less than or equal to the mapping radius. }
{$IFDEF PATCH_GH}
				if Cheat_Fix_SolveLine then begin
					Obs := CalcObscurement( P.X , P.Y , MZ , X , Y , TerrMan[ GB^.Map[ X , Y ].Terr ].Altitude , GB );
				end else begin
					Obs := CalcObscurement( X , Y , TerrMan[ GB^.Map[ X , Y ].Terr ].Altitude , P.X , P.Y , MZ , GB );
				end;
{$ELSE PATCH_GH}
				Obs := CalcObscurement( X , Y , TerrMan[ GB^.Map[ X , Y ].Terr ].Altitude , P.X , P.Y , MZ , GB );
{$ENDIF PATCH_GH}
				if (( Range( P.X , P.Y , X , Y ) + Obs ) <= R ) and ( Obs <> -1 ) then begin
					GB^.Map[X,Y].Visible := True;
					RedrawTile( GB , X , Y );
				end;
			end;
		end;
	end;
end;

Procedure VisionCheck( GB: GameBoardPtr; Mek: GearPtr );
	{ Perform a sensor check for MEK. It might spot hidden enemy }
	{ units; it may get spotted by enemy units itself. }
var
	M2: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	if ( Mek = Nil ) or ( not GearOperational( Mek ) ) or ( not OnTheMap( Mek ) ) then exit;

	{ Start by assuming that the mek will be hidden after this. }
	{ Strip all of its visibility tokens. }
	StripNAtt( Mek , NAG_Visibility );

	M2 := GB^.Meks;
	while M2 <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < M2^.G) then begin
{$ENDIF PATCH_GH}
		{ We are only interested in this other mek if it's an }
		{ enemy of the one we're checking. }
		if OnTheMap( M2 ) then begin
			if not MekCanSeeTarget( gb , Mek , M2 ) then begin

				{ If this enemy mecha has not yet been spotted, }
				{ there's a chance it will become visible. }
				if IsMasterGear( M2 ) and CheckLOS( GB , Mek , M2 ) then begin
					{ M2 has just been spotted. }
					RevealMek( GB , M2 , Mek );
				end;
			end;

			{ There is also a chance that M2 might spot Mek. }
			if IsMasterGear( M2 ) and GearOperational( M2 ) and not MekCanSeeTarget( GB , M2 , Mek ) then begin
				if CheckLOS( GB , M2 , Mek ) then RevealMek( GB , Mek , M2 );
			end;
		end;
{$IFDEF PATCH_GH}
		end;
{$ENDIF PATCH_GH}

		M2 := M2^.Next;
	end;

	if (NAttValue( Mek^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam) or (NAttValue( Mek^.NA , NAG_Location , NAS_Team ) = NAV_LancemateTeam) then begin
		CheckVisibleArea( GB , Mek );
	end;

	{ Redraw the spotter's tile. }
	RedrawTile( gb , Mek );
end;


Procedure DeployMek( GB: GameBoardPtr; Mek: GearPtr; PutOnMap: Boolean );
	{ Stick the provided MEK onto the game board. Assign a UID, }
	{ and set the default orders for MEK's team. }
	{ PRECONDITION: Mek must be unlinked. }
var
	Team: GearPtr;
	P: Point;
{$IFDEF PATCH_GH}
	Home: GearPtr;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) or (NIL = GB) then Exit;
{$ELSE PATCH_GH}
	if ( GB = Nil ) or ( Mek = Nil ) then Exit;
{$ENDIF PATCH_GH}

	{ Find the team for this model. }
	Team := LocateTeam( GB , NAttValue( Mek^.NA , NAG_Location , NAS_Team ) );

	{ Gear up the mek. }
	if PutOnMap then GearUp( Mek );

	{ Determine the X and Y values for everything. }
	{ If, according to the PutOnMap parameter, we aren't supposed to put }
	{ this model on the map, set X and Y to 0. }
	{ If the model has X and Y already defined within the map boundaries, }
	{ place it on the map at its specified location. }
	{ Otherwise, determine a good spot to place this mek based upon its }
	{ assigned team. }
{$IFDEF PATCH_GH}
	Home := NIL;
	if ( '' <> SAttValue( Mek^.SA , 'HOME' ) ) and ( NIL <> GB^.Scene ) then begin
		Home := SeekGearByName( GB^.Scene , SAttValue( Mek^.SA , 'HOME' ) );
	end;
	P.X := NAttValue( Mek^.NA , NAG_Location , NAS_X );
	P.Y := NAttValue( Mek^.NA , NAG_Location , NAS_Y );
{$ENDIF PATCH_GH}
	if not PutOnMap then begin
		SetNAtt( mek^.NA , NAG_Location , NAS_X , 0 );
		SetNAtt( mek^.NA , NAG_Location , NAS_Y , 0 );

{$IFDEF PATCH_GH}
	end else if ( OnTheMap( Mek ) and ( NIL <> Home )
	and ( ( Home^.Stat[ STAT_XPos ] <= P.X ) and ( P.X <= ( Home^.Stat[ STAT_XPos ] + Home^.Stat[ Stat_MFWidth ] ) ) )
	and ( ( Home^.Stat[ STAT_YPos ] <= P.Y ) and ( P.Y <= ( Home^.Stat[ STAT_YPos ] + Home^.Stat[ Stat_MFHeight ] ) ) )
	) then begin
		{ It has been already located. }
		;
{$ENDIF PATCH_GH}
	end else if OnTheMap( Mek ) and ( SAttValue( Mek^.SA , 'HOME' ) = '' ) then begin
		{ Just set a random direction. }
		SetNAtt( mek^.NA , NAG_Location , NAS_D , Random( 8 ) );

	end else begin
		P := FindDeploymentSpot( GB , Mek );
		SetNAtt( mek^.NA , NAG_Location , NAS_X , P.X );
		SetNAtt( mek^.NA , NAG_Location , NAS_Y , P.Y );
		SetNAtt( mek^.NA , NAG_Location , NAS_D , Random( 8 ) );
	end;


	{ Assign a unique ID for this model. }
	SetNAtt( mek^.NA , NAG_EpisodeData, NAS_UID, MaxIdTag( GB^.Meks , NAG_EpisodeData, NAS_UID ) + 1 );

	{ Stick mek on board. }
{$IFDEF PATCH_GH}
	Mek^.Next := NIL;
	AppendGear( GB^.Meks, Mek );
{$ELSE PATCH_GH}
	Mek^.Next := gb^.Meks;
	gb^.Meks := Mek;
{$ENDIF PATCH_GH}

	{ Set default orders. }
{$IFDEF PATCH_GH}
	if (NIL <> Team) and (GG_DisposeGear < Team^.G) then begin
{$ELSE PATCH_GH}
	if Team <> Nil then begin
{$ENDIF PATCH_GH}
		SetNAtt( Mek^.NA , NAG_EpisodeData , NAS_Orders , Team^.Stat[ STAT_TeamOrders ] );
	end;
end;

Procedure DeployMek( GB: GameBoardPtr; Mek,Pilot: GearPtr; Team: Integer );
	{ Insert the supplied pilot into the supplied mecha, then insert both }
	{ into the provided scenario. Supply UIDs to mek and pilot. }
	{ PRECONDITION: Mek and Pilot are both unlinked gears. }
begin
	if ( GB = Nil ) or ( Mek = Nil ) or ( Pilot = Nil ) then Exit;
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
	if (NIL = Pilot) or (Pilot^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Set the correct values for everything. }
	SetNAtt( mek^.NA , NAG_Location , NAS_Team , Team );

	SetNAtt( Pilot^.NA , NAG_Location , NAS_Team , Team );
	SetNAtt( Pilot^.NA , NAG_EpisodeData, NAS_UID, MaxIdTag( GB^.Meks , NAG_EpisodeData, NAS_UID ) + 2 );

	{ Locate cockpit, and install pilot. }
	if not BoardMecha( Mek , Pilot ) then begin
		{ PILOT couldn't be installed in MEK. }
		{ Stick PILOT off the map. }
		DeployMek( GB , Pilot, False );
	end;

	DeployMek( gb , Mek , True );
end;

Function LocateMekByUID( GB: GameBoardPtr; UID: Integer ): GearPtr;
	{ Search through the list of mecha associated with this scenario, and }
	{ return the mecha with the given Unique ID. Return Nil if it can't }
	{ be found. }
begin
	{ Error check - UID 0 is impossible. }
	if UID = 0 then Exit( Nil );

	{ Return whatever we found. }
	LocateMekByUID := SeekGearByIDTag( GB^.Meks , NAG_EpisodeData , NAS_UID , UID );
end;

Function TimeString( ComTime: LongInt ): String;
	{ Create a string to express the time listed in COMTIME. }
var
	msg: String;
	S,M,H,D: LongInt;	{ Seconds, Minutes, Hours, Days }
begin
	S := ComTime mod 60;
	M := ( ComTime div 60 ) mod 60;
	H := ( ComTime div AP_Hour ) mod 24;
	D := ComTime div AP_Day;

{$IFDEF PATCH_I18N}
	msg := ReplaceHash( I18N_MsgString('TIMESTRING'), BStr( D ), WideStr( H, 2 ), WideStr( M , 2 ), WideStr( S , 2 ) );
{$ELSE PATCH_I18N}
	msg := Bstr( H ) + ':' + WideStr( M , 2 ) + ':' + WideStr( S , 2 ) + MsgString( 'CLOCK_days' ) + BStr( D );
{$ENDIF PATCH_I18N}
	TimeString := msg;
end;

Procedure UpdateCombatDisplay( GB: GameBoardPtr );
	{ Just update the screen; don't redraw everything. }
begin
{$IFDEF PATCH_HEART}
	if ( NIL = GB ) then begin
	end else begin
		CMessage( TimeString( GB^.ComTime ) , ZONE_Clock , White );
	end;
{$ELSE PATCH_HEART}
	CMessage( TimeString( GB^.ComTime ) , ZONE_Clock , White );
{$ENDIF PATCH_HEART}
end;

Procedure GFCombatDisplay( GB: GameBoardPtr );
	{ Do each of the standard GearFitr display elements. }
begin
	DisplayMap( GB );
	UpdateCombatDisplay( GB );
end;

Procedure FocusOnMek( GB: GameBoardPtr; Mek: GearPtr );
	{ Recenter the display on the indicated mecha. }
var
	P: Point;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	if ( Mek <> Nil ) and ( GB <> Nil ) then begin
		P := GearCurrentLocation( Mek );
		RecenterDisplay( P.X , P.Y );
		DisplayMap( gb );
	end;
end;

Function ProcessShotAnimation( GB: GameBoardPtr; var AnimList,AnimOb: GearPtr ): Boolean;
	{ Process this shot. Return TRUE if the missile }
	{ is visible on the screen, FALSE otherwise. }
	{ V = Timer }
	{ Stat 1 , 2 , 3 -> X1 , Y1 , Z1 }
	{ Stat 4 , 5 , 6 -> X2 , Y2 , Z2 }
const
	X1 = 1;
	Y1 = 2;
	X2 = 4;
	Y2 = 5;
var
	P: Point;
begin
{$IFDEF PATCH_GH}
	if (NIL = AnimList) or (AnimList^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = AnimOb) or (AnimOb^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}

	{ Redraw the tile from last time. }
	P := SolveLine( AnimOb^.Stat[ X1 ] , AnimOb^.Stat[ Y1 ] , AnimOb^.Stat[ X2 ] , AnimOb^.Stat[ Y2 ] , AnimOb^.V );
	RedrawTile( GB , P.X , P.Y );

	{ Increase the counter, and find the next spot. }
	Inc( AnimOb^.V );
	P := SolveLine( AnimOb^.Stat[ X1 ] , AnimOb^.Stat[ Y1 ] , AnimOb^.Stat[ X2 ] , AnimOb^.Stat[ Y2 ] , AnimOb^.V );
{$IFDEF PATCH_JPSSDL}
	if SkipAnim and (AnimOb^.V > 1) then begin
		P.X := AnimOb^.Stat[ X2 ];
		P.Y := AnimOb^.Stat[ Y2 ];
	end;
{$ENDIF PATCH_JPSSDL}

	{ If this is the destination point, then we're done. }
	if ( P.X = AnimOb^.Stat[ X2 ] ) and ( P.Y = AnimOb^.Stat[ Y2 ] ) then begin
		RemoveGear( AnimList , ANimOb );
		P.X := 0;

	{ If this is not the destination point, draw the missile. }
	end else begin
		{Display bullet...}
		DrawMapImage( '+' , P.X , p.Y , LightRed );
{$IFDEF PATCH_JPSSDL}
		if SkipAnim then DrawMapImage( '+' , AnimOb^.Stat[X2] , AnimOb^.Stat[Y2] , LightRed );
{$ENDIF PATCH_JPSSDL}
	end;

	ProcessShotAnimation := OnTheScreen( P.X , P.Y );
end;

Function ProcessPointAnimation( GB: GameBoardPtr; var AnimList,AnimOb: GearPtr ): Boolean;
	{ Process this effect. Return TRUE if the blast }
	{ is visible on the screen, FALSE otherwise. }
	{ V = Timer }
	{ Stat 1 , 2 , 3 -> X , Y , Z }
const
	X = 1;
	Y = 2;
	NumBlastColor = 8;
{$IFDEF ASCII_SDL}
	BlastColor: Array [0..NumBlastColor-1] of TSDL_Color = (
		( r:192; g:  0; b:  0 ),	{ Red, }
		( r:255; g:  0; b:  0 ),	{ LightRed, }
		( r:255; g:  0; b:  0 ),	{ LightRed, }
		( r:255; g:  0; b:255 ),	{ LightMagenta, }
		( r:255; g:255; b:127 ),	{ Yellow, }
		( r:255; g:255; b:255 ),	{ White, }

		( r:255; g:  0; b:  0 ),	{ LightRed, }
		( r:255; g:255; b:127 )		{ Yellow }
	);
{$ELSE ASCII_SDL}
	BlastColor: Array [0..NumBlastColor-1] of Byte = (
		Red, LightRed, LightRed, LightMagenta, Yellow, White,
		LightRed, Yellow
	);
{$ENDIF ASCII_SDL}
var
	gfx: Char;
{$IFDEF ASCII_SDL}
	c: TSDL_Color;
{$ELSE ASCII_SDL}
	c: Byte;
{$ENDIF ASCII_SDL}
	it: Boolean;
begin
{$IFDEF PATCH_GH}
	if (NIL = AnimList) or (AnimList^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = AnimOb) or (AnimOb^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}

	if AnimOb^.V < 3 then begin
		case AnimOb^.S of
		GS_DamagingHit: begin
				gfx := '*';
				c := LightRed;
				end;
		GS_ArmorDefHit: begin
				gfx := '*';
				c := DarkGray;
				end;
		GS_Parry:	begin
				gfx := '!';
				c := DarkGray;
				end;
		GS_Dodge:	begin
				gfx := '-';
				c := DarkGray;
				end;
		GS_Backlash:	begin
				gfx := '*';
				c := Yellow;
				end;
		GS_AreaAttack:	begin
				gfx := '*';
				c := BlastColor[ Random( NumBlastColor ) ];
				end;
		end;
		DrawMapImage( gfx , AnimOb^.Stat[ X ] , AnimOb^.Stat[ Y ] , c );

		{ Increment the counter. }
		Inc( AnimOb^.V );
{$IFDEF PATCH_JPSSDL}
		if SkipAnim then AnimOb^.V := 10;
{$ENDIF PATCH_JPSSDL}

		it := OnTheScreen( AnimOb^.Stat[ X ] , AnimOb^.Stat[ Y ] );
	end else begin
		RedrawTile( GB , AnimOb^.Stat[ X ] , AnimOb^.Stat[ Y ] );
		RemoveGear( AnimList , AnimOb );
		it := False;
	end;

	ProcessPointAnimation := it;
end;

Procedure ProcessAnimations( GB: GameBoardPtr; var AnimList: GearPtr );
	{ Display all the queued animations, deleting them as we go along. }
var
	AnimOb,A2: GearPtr;
	DelayThisFrame,PointDelay: Boolean;
begin
	{ Keep processing until we run out of animation objects. }
	while AnimList <> Nil do begin
		AnimOb := AnimList;

		{ Assume there'll be no animation delay, unless }
		{ otherwise requested. }
		DelayThisFrame := False;

		while AnimOb <> Nil do begin
			A2 := AnimOb^.Next;
{$IFDEF PATCH_GH}
			if (GG_DisposeGear < AnimOb^.G) then begin
{$ENDIF PATCH_GH}
			{ Call a routine based upon the type of }
			{ animation requested. }
			case AnimOb^.S of

			GS_Shot: PointDelay := ProcessShotAnimation( GB , AnimList , AnimOb ) or DelayThisFrame;
			GS_DamagingHit: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );
			GS_ArmorDefHit: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );
			GS_Parry: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );
			GS_Dodge: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );
			GS_Backlash: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );
			GS_AreaAttack: PointDelay := ProcessPointAnimation( GB , AnimList , AnimOb );

			{ If no routine was found to deal with the animation }
			{ requested, just delete the gear. }
			else RemoveGear( AnimList , AnimOb );
			end;

			DelayThisFrame := DelayThisFrame or PointDelay;
{$IFDEF PATCH_GH}
			end;
{$ENDIF PATCH_GH}
			{ Move to the next animation. }
			AnimOb := A2;
		end;

		{ Delay the animations, if appropriate. }
{$IF DEFINED(ASCII_SDL)}
		ascii_sdl.DoFlip;
		if ( 0 < FrameDelay ) and DelayThisFrame then ascii_sdl.Delay(FrameDelay);
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		if ( FrameDelay > 0 ) and DelayThisFrame then w32crt.Delay(FrameDelay);
{$ELSE}
		if ( FrameDelay > 0 ) and DelayThisFrame then Delay(FrameDelay);
{$ENDIF}
	end;
end;

Function DisplayEffectAnimations( GB: GameBoardPtr; N: Integer ): Boolean;
	{ Display all the animations stored for sequence slice N. }
	{ Return TRUE if an animation was found, or FALSE otherwise. }
var
	A: SAttPtr;
	T: Integer;
	AnimFound: Boolean;
	AnimList,AnimItem: GearPtr;
	AnimLabel,AnimCode: String;
begin
	A := ATTACK_History;
	AnimFound := False;
	AnimList := Nil;

	AnimLabel := SATT_Anim_Direction + BStr( N ) + '_';

	{ Start by creating the animation list. }
	while A <> Nil do begin
		if HeadMatchesString( AnimLabel , A^.Info ) then begin
			AnimFound := True;

			{ Insert animation handling code here. }
			AnimItem := AddGear( AnimList , Nil );
			AnimCode := RetrieveAString( A^.Info );
			AnimItem^.S := ExtractValue( AnimCode );

			T := 1;
			while ( AnimCode <> '' ) and ( T <= NumGearStats ) do begin
				AnimItem^.Stat[ T ] := ExtractValue( AnimCode );
				Inc( T );
			end;
		end;
		A := A^.Next;
	end;

	{ Process each animation. }
	ProcessAnimations( GB , AnimList );

	DisplayEffectAnimations := AnimFound;
end;


Procedure DisplayConsoleHistory( GB: GameBoardPtr );
	{ Display the console history, then restore the display. }
var
	SL: SAttPtr;
{$IFDEF PATCH_I18N}
	MaxWidth: Integer;
{$ENDIF PATCH_I18N}
begin
{$IFDEF PATCH_GH}
	MoreText( Console_History , -1 );
{$ELSE PATCH_GH}
	MoreText( Console_History , MoreHighFirstLine( Console_History ) );
{$ENDIF PATCH_GH}
	GFCombatDisplay( GB );

{$IFDEF PATCH_GH}
	RedrawConsole;
{$ELSE PATCH_GH}
	{ Restore the console display. }
  {$IF DEFINED(ASCII_SDL)}
	ascii_sdl.GotoXY( ScreenZone[ ZONE_Dialog , 1 ] , ScreenZone[ ZONE_Dialog , 2 ] -1 );
	ascii_sdl.TextColor( Green );
  {$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.GotoXY( ScreenZone[ ZONE_Dialog , 1 ] , ScreenZone[ ZONE_Dialog , 2 ] -1 );
	w32crt.TextColor( Green );
  {$ELSE}
	GotoXY( ScreenZone[ ZONE_Dialog , 1 ] , ScreenZone[ ZONE_Dialog , 2 ] -1 );
	TextColor( Green );
  {$ENDIF}
  {$IFDEF PATCH_I18N}
	MaxWidth := ScreenZone[ZONE_Dialog,3] - ScreenZone[ZONE_Dialog,1];
  {$ENDIF PATCH_I18N}
	SL := RetrieveSAtt( Console_History , NumSAtts( Console_History ) - ScreenRows + ScreenZone[ ZONE_Dialog , 2 ] );
	if SL = Nil then SL := Console_History;
	while SL <> Nil do begin
  {$IF DEFINED(ASCII_SDL)}
		ascii_sdl.writeln;
  {$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.WriteLn;
  {$ELSE}
		writeln;
  {$ENDIF}
  {$IFDEF PATCH_I18N}
		WriteMBCharStr( SL^.Info, MaxWidth );
  {$ELSE PATCH_I18N}
		write( SL^.Info );
  {$ENDIF PATCH_I18N}
		SL := SL^.Next;
	end;
{$ENDIF PATCH_GH}
end;

Procedure ProcessMovement( GB: GameBoardPtr; Mek: GearPtr );
	{ Call the LOCALE movement routine, then update the display }
	{ here if need be. }
var
	result,Team: Integer;
	X,Y: LongInt;
	msg: String;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Store the initial position of the mek. }
	X := NAttValue( Mek^.NA , NAG_Location , NAS_X );
	Y := NAttValue( Mek^.NA , NAG_Location , NAS_Y );

	{ Call the movement procedure, and store the result. }
	result := EnactMovement( GB , Mek );

	{ Depending upon what happened, update the display. }
	if result > 0 then begin
		{ Update the display. }
		RedrawTile( GB , X , Y );
		RedrawTile( GB , Mek );

		{ Check for previously unseen enemies. }
		if OnTheMap( NAttValue( Mek^.NA , NAG_Location , NAS_X ) , NAttValue( Mek^.NA , NAG_Location , NAS_Y ) ) then VisionCheck( GB , Mek )
		{ Print message if mek has fled the battle. }
		else begin
{$IFDEF PATCH_I18N}
			DialogMSG( ReplaceHash( I18N_MsgString('ProcessMovement','Left'), PilotName(Mek)) );
{$ELSE PATCH_I18N}
			DialogMSG( PilotName( Mek ) + ' has left this area.');
{$ENDIF PATCH_I18N}

			{ Set trigger here. }
			Team := NAttValue( Mek^.NA , NAG_Location , NAS_Team );
			SetTrigger( GB , TRIGGER_NumberOfUnits + BStr( Team ) );
			SetTrigger( GB , TRIGGER_UnitEliminated2 + BStr( NAttValue( Mek^.NA , NAG_EpisodeData , NAS_UID ) ) );
		end;

	end else if result = EMR_Crash then begin
		{ Update the display. }
		RedrawTile( GB , X , Y );
		RedrawTile( GB , Mek );

{$IFDEF PATCH_I18N}
		if Mek^.G = GG_Character then begin
			msg := I18N_MsgString('ProcessMovement','Fall');
		end else begin
			msg := I18N_MsgString('ProcessMovement','Crash');
		end;
		DialogMsg( ReplaceHash( msg, GearName(Mek), BStr(DAMAGE_DamageDone) ) );
{$ELSE PATCH_I18N}
		if Mek^.G = GG_Character then begin
			msg := ReplaceHash( MsgString( 'PROCESSMOVEMENT_Fall' ) , GearName( Mek ) );
		end else begin
			msg := ReplaceHash( MsgString( 'PROCESSMOVEMENT_Crash' ) , GearName( Mek ) );
		end;
		DialogMsg( ReplaceHash( msg , BStr( DAMAGE_DamageDone ) ) );
{$ENDIF PATCH_I18N}
	end;
end;


Procedure WriteCampaign( Camp: CampaignPtr; var F: Text );
	{ Output the supplied campaign and all appropriate data to disk. }
var
	Frz: FrozenLocationPtr;
{$IFDEF PATCH_HEART}
	G: GearPtr;
{$ENDIF PATCH_HEART}
begin
	{ Output GameBoard. }
{$IF DEFINED(ASCII_SDL)}
	system.writeln( F , Camp^.GB^.ComTime );
	system.writeln( F , Camp^.GB^.Scale );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	system.writeln( F , Camp^.GB^.ComTime );
	system.writeln( F , Camp^.GB^.Scale );
{$ELSE}
	writeln( F , Camp^.GB^.ComTime );
	writeln( F , Camp^.GB^.Scale );
{$ENDIF}
	WriteMap( Camp^.GB^.Map , F );

	{ Can't output the scene gear directly, since it'll be outputted }
	{ with the rest of SOURCE later on. Output its reference number. }
{$IF DEFINED(ASCII_SDL)}
	system.writeln( F , FindGearIndex( Camp^.Source , Camp^.GB^.Scene ) );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	system.writeln( F , FindGearIndex( Camp^.Source , Camp^.GB^.Scene ) );
{$ELSE}
	writeln( F , FindGearIndex( Camp^.Source , Camp^.GB^.Scene ) );
{$ENDIF}
{$IFDEF DEBUG}
	ErrorMessage_fork( 'TRACE: WriteCampaign(): '
		+ BStr(FindGearIndex( Camp^.Source , Camp^.GB^.Scene ))
		+ '/' + BStr(CountGearIndex( Camp^.Source, False, '' ))
		+ '/' + BStr(CountGearIndex( Camp^.Source, True, '' ))
		+ ': ' + Camp^.GB^.Scene^.SA^.info );
	DialogMsg( 'TRACE: WriteCampaign(): '
		+ BStr(FindGearIndex( Camp^.Source , Camp^.GB^.Scene ))
		+ '/' + BStr(CountGearIndex( Camp^.Source, False, '' ))
		+ '/' + BStr(CountGearIndex( Camp^.Source, True, '' ))
		+ ': ' + Camp^.GB^.Scene^.SA^.info );
{$ENDIF DEBUG}

	{ Output map contents. }
	WriteCGears( F , Camp^.GB^.Meks );
{$IFDEF PATCH_HEART}
	G := Camp^.GB^.Meks;
	while ( NIL <>  G ) do begin
		if ( NIL <> G^.QRL ) then begin
			QRL_hibernate( G , Camp^.GB );
		end;
		G := G^.Next;
	end;
{$ENDIF PATCH_HEART}

	{ Output frozen maps. }
	Frz := Camp^.Maps;
	while Frz <> Nil do begin
{$IF DEFINED(ASCII_SDL)}
		system.writeln( F , FROZEN_MAP_CONTINUE );
		system.writeln( F , Frz^.Name );
{$ELSEIF DEFINED(GUIMSWINMODE)}
		system.writeln( F , FROZEN_MAP_CONTINUE );
		system.writeln( F , Frz^.Name );
{$ELSE}
		writeln( F , FROZEN_MAP_CONTINUE );
		writeln( F , Frz^.Name );
{$ENDIF}
		WriteMap( Frz^.Map , F );
		Frz := Frz^.Next;
	end;

	{ Output frozen map sentinel marker. }
{$IF DEFINED(ASCII_SDL)}
	system.writeln( F , FROZEN_MAP_SENTINEL );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	system.writeln( F , FROZEN_MAP_SENTINEL );
{$ELSE}
	writeln( F , FROZEN_MAP_SENTINEL );
{$ENDIF}

	{ Output SOURCE. }
	WriteCGears( F , Camp^.Source );
end;

Function ReadCampaign( var F: Text ): CampaignPtr;
	{ Input the campaign and all appropriate data from disk. }
var
	Camp: CampaignPtr;
	SceneIndex: LongInt;
	N: Integer;
	Frz: FrozenLocationPtr;
{$IFDEF DEBUG}
	msg: String;
{$ENDIF DEBUG}
begin
	{ Allocate the campaign and the gameboard. }
	Camp := NewCampaign;
	Camp^.GB := NewMap;

	readln( F , Camp^.GB^.ComTime );
	Camp^.ComTime := Camp^.GB^.ComTime;
	readln( F , Camp^.GB^.Scale );
	Camp^.GB^.Map := ReadMap( F );

	{ Read the index of this game board's SCENE gear, and }
	{ remember to set it in the GB structure after SOURCE is loaded. }
	readln( F , SceneIndex );

	{ Read the list of map contents. }
	Camp^.GB^.Meks := ReadCGears( F );

	{ Read the frozen maps. }
	repeat
		ReadLn( F , N );

		if N = FROZEN_MAP_CONTINUE then begin
			Frz := CreateFrozenLocation( Camp^.Maps );
			ReadLn( F , Frz^.Name );
			Frz^.Map := ReadMap( F );
		end;
	until N = FROZEN_MAP_SENTINEL;

	{ Read the source, and set the gameboard's scene. }
	Camp^.Source := ReadCGears( F );
{$IFDEF DEBUG}
	Camp^.GB^.Scene := LocateGearByNumber( Camp^.Source , SceneIndex, False, 0, '' );
	msg := BStr(FindGearIndex( Camp^.Source , Camp^.GB^.Scene ));
	msg := msg + '/' + BStr(CountGearIndex( Camp^.Source, False, '' ));
	msg := msg + '/' + BStr(CountGearIndex( Camp^.Source, True, '' ));
	if (NIL <> Camp^.GB^.Scene^.SA) then begin
		msg := msg + ': ' + Camp^.GB^.Scene^.SA^.info;
	end else begin
		msg := msg + ': (NIL)';
	end;
	ErrorMessage_fork( 'TRACE: ReadCampaign(): ' + msg );
	DialogMsg( 'TRACE: ReadCampaign(): ' + msg );
{$ELSE DEBUG}
	Camp^.GB^.Scene := LocateGearByNumber( Camp^.Source , SceneIndex );
{$ENDIF DEBUG}

	{ Return the restored campaign structure. }
	ReadCampaign := Camp;
end;

Procedure BeginTurn( GB: GameBoardPtr; M: GearPtr );
	{ Time to start the turn. }
var
	A: Char;
begin
	SetupMemoDisplay;
	GameMsg( 'Begin ' + PilotName( M ) + ' turn' , ZONE_MemoText , InfoHilight );
	EndOfGameMoreKey;
end;



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: conmap.pp');
{$ENDIF DEBUG}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: conmap.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
