{$IFDEF PATCH_HEART}
unit qrl;
{ Quasi Robot Language }

interface

uses sysutils,
{$IFDEF LUAJIT51}
	luajit51,
{$ELSE LUAJIT51}
	lua53,
{$ENDIF LUAJIT51}
	gears_base, gears, locale;

Procedure CallQRLdebugger( Mek: GearPtr; Mode: Boolean );
Function QRL_start( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
Function QRL_resume( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
Function QRL_hibernate( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
Function QRL_destroy( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
Function QRL_stop( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
Function GetQRLInput( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
{$IFDEF PATCH_GH_PARANOID_SAFER}
Procedure QRL_close( QRL: PLua_State; QRL_Name: String );
{$ENDIF PATCH_GH_PARANOID_SAFER}
Procedure QRL_SetFile( GB: GameBoardPtr; Mek: GearPtr; Name: String );



implementation

uses errmsg,
{$IFDEF DEBUG}
	pseudosmartpointer,
{$ENDIF DEBUG}
	ui4gh, ability, arenaplay, gearutil, texutil,
	qrl_init, qrl_glue,
{$IFDEF SDLMODE}
	sdlgfx
{$ELSE SDLMODE}
	context
{$ENDIF SDLMODE}
{$IFDEF SDL2}
	,qrl_viewer
{$ELSE SDL2}
{$ENDIF SDL2}
	;

const
	QRL_Max_Stack: Integer = 128;

	QRL_FunctionName_Pre = 'qrl_chip';
	QRL_FunctionName_start = '_start';
	QRL_FunctionName_resume = '_resume';
	QRL_FunctionName_hibernate = '_hibernate';
	QRL_FunctionName_destroy = '_destroy';
	QRL_FunctionName_stop = '_stop';
	QRL_FunctionName_initialPoint = '0000';

	LuaDebuggerFunctionName = 'call_debugger';

var
	DEBUG_NowInTheQRL : Boolean = False;


Procedure CallQRLdebugger( Mek: GearPtr; Mode: Boolean );
var
	pmsg: PChar;
	ret: Integer;
begin
	if ( NIL = Mek^.QRL ) then begin
		ErrorMessage_fork('ERROR: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') ) failed. Not initialized.' );
		exit;
	end;

	if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
		pmsg := QuickPCopy( LuaDebuggerFunctionName );
		lua_getglobal( Mek^.QRL , pmsg );

		if Mode then begin
			ErrorMessage_fork('TRACE: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') , true ).' );
			DialogMsg('TRACE: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') , true ).' );
			lua_pushinteger( Mek^.QRL , 1 );
		end else begin
			ErrorMessage_fork('TRACE: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') , false ).' );
			DialogMsg('TRACE: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') , false ).' );
			lua_pushinteger( Mek^.QRL , 0 );
		end;

		DEBUG_NowInTheQRL := True;
		ret := lua_pcall( Mek^.QRL , 1 , 0 , 0 );
		DEBUG_NowInTheQRL := False;
		sysutils.StrDispose (pmsg);

		if ( 0 <> ret ) then begin
			ErrorMessage_fork('ERROR: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_pcall(' + LuaDebuggerFunctionName + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
			DialogMsg('ERROR: CallQRLdebugger( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_pcall(' + LuaDebuggerFunctionName + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
{$IFDEF DEBUG}
			QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
			lua_pop( Mek^.QRL , 1 );
		end;
	end;
end;


Function QRL_open( var QRL: PLua_State; QRL_Name: String ): Boolean;
var
	userfilename: String;
	pmsg: PChar;
	ret: Integer;
begin
	QRL := luaL_newstate;
	if ( NIL = QRL ) then begin
		ErrorMessage_fork('ERROR: luaL_newstate(' + QRL_Name + ') failed.');
		exit (False);
	end;

	if not lua_checkstack( QRL , QRL_Max_Stack ) then begin
		ErrorMessage_fork('ERROR: lua_checkstack(' + QRL_Name + ') failed : Can not allocate a stack.' );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( QRL );
{$ENDIF DEBUG}
		QRL_close( QRL , QRL_Name );
		QRL := NIL;
		exit (False);
	end;

	luaL_openlibs( QRL );

	userfilename := QRL_UserDirectory + QRL_Name + QRL_ScriptFileExtName;
	if FileExists( userfilename ) then begin
		pmsg := QuickPCopy( userfilename );
	end else begin
		pmsg := QuickPCopy( QRL_SystemDirectory + QRL_Name + QRL_ScriptFileExtName );
	end;
	ret := luaL_dofile( QRL , pmsg );
	sysutils.StrDispose (pmsg);
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: luaL_dofile(' + QRL_Name + ') failed : ' + lua_tostring( QRL , -1 ) );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( QRL );
{$ENDIF DEBUG}
		lua_pop( QRL , 1 );
		QRL_close( QRL , QRL_Name );
		QRL := NIL;
		exit (False);
	end;

{$IFDEF DEBUG}
	if DEBUG_TraceQRL and ( NIL <> QRL ) then begin
		ErrorMessage_fork('TRACE: QRL_open( ' + QRL_Name + ' ).' );
	end;
{$ENDIF DEBUG}

	QRL_open := True;
end;

Procedure QRL_close( QRL: PLua_State; QRL_Name: String );
begin
{$IFDEF DEBUG}
	if DEBUG_TraceQRL and ( NIL <> QRL ) then begin
		ErrorMessage_fork('TRACE: QRL_close( ' + QRL_Name + ' ).' );
	end;
{$ENDIF DEBUG}

	if ( NIL = QRL ) then begin
		exit;
	end;

	lua_close( QRL );
end;


Procedure QRL_SetFile( GB: GameBoardPtr; Mek: GearPtr; Name: String );
var
	S: SAttPtr;
begin
	if ( NIL <> Mek^.QRL ) then begin
		QRL_destroy( Mek , GB );
	end;
	SetSAtt( Mek^.SA , QRL_FileName_TAG + ' <' + Name + '>' );
	SetSAtt( Mek^.SA , QRL_Ptr_TAG + ' <' + '>' );
	SetSAtt( Mek^.SA , QRL_ChipMode_TAG + ' <' + BStr( 0 ) + '>' );
	SetSAtt( Mek^.SA , QRL_ChipMode_LoopCount_TAG + ' <' + BStr( 0 ) + '>' );

	repeat
		S := FindHeadMatch( QRL_Telecommunication_TAG , Mek^.SA , 1 );
		if (NIL <> S) then begin
			RemoveSAtt( Mek^.SA , S );
		end;
	until (NIL = S);
end;


Function QRL_begin( GB: GameBoardPtr; Mek: GearPtr; Func: String ): Boolean;
var
	QRL_Name: String;
	QRL_Ptr: String;
	pmsg: PChar;
	ret: Integer;
begin
{$IFDEF DEBUG}
	if DEBUG_TraceQRL then begin
		ErrorMessage_fork('TRACE: QRL_begin( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ).' );
	end;
{$ENDIF DEBUG}

	SetSAtt( Mek^.SA , QRL_Pilot_TAG + ' <' + PilotName( Mek ) + '>' );

	if ( NIL = Mek^.QRL ) then begin
		QRL_Name := SAttValue( Mek^.SA , QRL_FileName_TAG );
		if ( 0 = Length( QRL_Name ) ) then begin
			QRL_Name := QRL_FilePrefix + QRL_File_Default;
		end;

		if not QRL_open( Mek^.QRL , QRL_Name ) then begin
			ErrorMessage_fork('ERROR: QRL_begin( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ) failed. Not initialized.' );
{$IFDEF DEBUG}
			QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
			exit (False);
		end;
{$IFDEF SDL2}
		if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
			QRL_Viewer_InitTracer( QRL_Name );
		end;
{$ELSE SDL2}
{$ENDIF SDL2}

		if not QRL_attach_glue( Mek^.QRL ) then begin
			ErrorMessage_fork('ERROR: QRL_begin( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ) failed at QRL_attach_glue().' );
{$IFDEF DEBUG}
			QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
			QRL_close( Mek^.QRL , QRL_Name );
			Mek^.QRL := NIL;
			exit (False);
		end;
	end;

	if DEBUG_EnableLuaDebugger then begin
		CallQRLdebugger( Mek , DEBUG_EnableLuaDebugger );
	end;

	QRL_Ptr := Func;

	pmsg := QuickPCopy( QRL_FunctionName_Pre + QRL_Ptr );
	lua_getglobal( Mek^.QRL , pmsg );
	if (NIL <> Mek) then begin
		lua_pushlightuserdata( Mek^.QRL , Mek );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	if (NIL <> GB) then begin
		lua_pushlightuserdata( Mek^.QRL , GB );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	DEBUG_NowInTheQRL := True;
	ret := lua_pcall( Mek^.QRL , 2 , 0 , 0 );
	DEBUG_NowInTheQRL := False;
	sysutils.StrDispose (pmsg);
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_begin( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ), lua_pcall(' + QRL_Ptr + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_pop( Mek^.QRL , 1 );
		exit (False);
	end;

	ret := lua_gettop( Mek^.QRL );
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_begin( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ), lua_gettop() mismatch at QRL_begin().' );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_settop( Mek^.QRL , 1 );
	end;

{$IFDEF SDL2}
	if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
		QRL_Viewer_StartTracer();
	end;
{$ELSE SDL2}
{$ENDIF SDL2}
	QRL_begin := True;
end;

Function QRL_end( GB: GameBoardPtr; Mek: GearPtr; Func: String ): Boolean;
var
	CampaignFileName: String;
	QRL_Name: String;
	QRL_Ptr: String;
	pmsg: PChar;
	S: SAttPtr;
	ret: Integer;
begin
{$IFDEF DEBUG}
	if DEBUG_TraceQRL then begin
		ErrorMessage_fork('TRACE: QRL_end( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ).' );
	end;
{$ENDIF DEBUG}

	if ( NIL = Mek^.QRL ) then begin
		ErrorMessage_fork('ERROR: QRL_end( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ) failed. Not initialized.' );
		exit (False);
	end;

	QRL_Ptr := Func;

	CampaignFileName := GB^.CampaignFileName;
	GB^.CampaignFileName := '';
	pmsg := QuickPCopy( QRL_FunctionName_Pre + QRL_Ptr );
	lua_getglobal( Mek^.QRL , pmsg );
	if (NIL <> Mek) then begin
		lua_pushlightuserdata( Mek^.QRL , Mek );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	if (NIL <> GB) then begin
		lua_pushlightuserdata( Mek^.QRL , GB );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	DEBUG_NowInTheQRL := True;
	ret := lua_pcall( Mek^.QRL , 2 , 0 , 0 );
	DEBUG_NowInTheQRL := False;
	sysutils.StrDispose (pmsg);
	GB^.CampaignFileName := CampaignFileName;
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_end( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ), lua_pcall(' + QRL_Ptr + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_pop( Mek^.QRL , 1 );
		exit (False);
	end;

	ret := lua_gettop( Mek^.QRL );
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_end( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + Func + ' ), lua_gettop() mismatch at QRL_end().' );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_settop( Mek^.QRL , 1 );
	end;

{$IFDEF SDL2}
	if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
		QRL_Viewer_StopTracer();
	end;
{$ELSE SDL2}
{$ENDIF SDL2}

	SetSAtt( Mek^.SA , QRL_Ptr_TAG + ' <' + '>' );
	SetSAtt( Mek^.SA , QRL_ChipMode_TAG + ' <' + BStr( 0 ) + '>' );
	SetSAtt( Mek^.SA , QRL_ChipMode_LoopCount_TAG + ' <' + BStr( 0 ) + '>' );
	repeat
		S := FindHeadMatch( QRL_Telecommunication_TAG , Mek^.SA , 1 );
		if (NIL <> S) then begin
			RemoveSAtt( Mek^.SA , S );
		end;
	until (NIL = S);

	QRL_Name := SAttValue( Mek^.SA , QRL_FileName_TAG );
	if ( 0 = Length( QRL_Name ) ) then begin
		QRL_Name := QRL_FilePrefix + QRL_File_Default;
	end;
	QRL_close( Mek^.QRL , QRL_Name );
	Mek^.QRL := NIL;

	QRL_end := True;
end;

Function QRL_start( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
begin
	QRL_start := QRL_begin( GB , Mek , QRL_FunctionName_start );
end;

Function QRL_resume( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
begin
	QRL_resume := QRL_begin( GB , Mek , QRL_FunctionName_resume );
end;

Function QRL_hibernate( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
var
	CampaignFileName: String;
	QRL_Ptr: String;
	pmsg: PChar;
	ret: Integer;
begin
{$IFDEF DEBUG}
	if DEBUG_TraceQRL then begin
		ErrorMessage_fork('TRACE: QRL_hibernate( Mek(' + GetMekName_for_Trace( Mek ) + ') ).' );
	end;
{$ENDIF DEBUG}

	if ( NIL = Mek^.QRL ) then begin
		ErrorMessage_fork('ERROR: QRL_hibernate( Mek(' + GetMekName_for_Trace( Mek ) + ') ) failed. Not initialized.' );
		exit (False);
	end;

	QRL_Ptr := QRL_FunctionName_hibernate;

	CampaignFileName := GB^.CampaignFileName;
	GB^.CampaignFileName := '';
	pmsg := QuickPCopy( QRL_FunctionName_Pre + QRL_Ptr );
	lua_getglobal( Mek^.QRL , pmsg );
	if (NIL <> Mek) then begin
		lua_pushlightuserdata( Mek^.QRL , Mek );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	if (NIL <> GB) then begin
		lua_pushlightuserdata( Mek^.QRL , GB );
	end else begin
		lua_pushnil( Mek^.QRL );
	end;
	DEBUG_NowInTheQRL := True;
	ret := lua_pcall( Mek^.QRL , 2 , 0 , 0 );
	DEBUG_NowInTheQRL := False;
	sysutils.StrDispose (pmsg);
	GB^.CampaignFileName := CampaignFileName;
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_hibernate( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_pcall(' + QRL_Ptr + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_pop( Mek^.QRL , 1 );
		exit (False);
	end;

	ret := lua_gettop( Mek^.QRL );
	if ( 0 <> ret ) then begin
		ErrorMessage_fork('ERROR: QRL_hibernate( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_gettop() mismatch at QRL_end().' );
{$IFDEF DEBUG}
		QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
		lua_settop( Mek^.QRL , 1 );
	end;

	QRL_hibernate := True;
end;

Function QRL_destroy( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
begin
	QRL_destroy := QRL_end( GB , Mek , QRL_FunctionName_destroy );
end;

Function QRL_stop( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
begin
	QRL_stop := QRL_end( GB , Mek , QRL_FunctionName_stop );
end;


Function GetQRLInput( Mek: GearPtr; GB: GameBoardPtr ): Boolean;
var
	QRL_ChipMode: Integer;
	QRL_ChipMode_LoopCount: Integer;
	QRL_Ptr: String;
	pmsg: PChar;
	s: String;
	ret: Integer;
begin
	if ( NIL = Mek^.QRL ) then begin
		ErrorMessage_fork('ERROR: GetQRLInput( Mek(' + GetMekName_for_Trace( Mek ) + ') ) failed. Not initialized.' );
		exit (False);
	end;

	SetSAtt( Mek^.SA , QRL_ChipMode_LoopCount_TAG + ' <' + BStr( 0 ) + '>' );
	while CheckMeks_EnemyAlive( GB^.Meks ) do begin
		SetSAtt( Mek^.SA , QRL_ChipMode_TAG + ' <' + BStr( QRL_ChipMode_DEFAULT ) + '>' );

		QRL_Ptr := SAttValue( Mek^.SA , QRL_Ptr_TAG );
		if ( 0 = Length( QRL_Ptr ) ) then begin
			QRL_Ptr := QRL_FunctionName_initialPoint;
		end;

{$IFDEF DEBUG}
		if (DEBUG_TraceQRL and (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team ))) then begin
			ErrorMessage_fork('TRACE: GetQRLInput( Mek(' + GetMekName_for_Trace( Mek ) + ') , ' + QRL_Ptr + ' ).' );
		end;
{$ENDIF DEBUG}

		pmsg := QuickPCopy( QRL_FunctionName_Pre + QRL_Ptr );
		lua_getglobal( Mek^.QRL , pmsg );
		if (NIL <> Mek) then begin
			lua_pushlightuserdata( Mek^.QRL , Mek );
		end else begin
			lua_pushnil( Mek^.QRL );
		end;
		if (NIL <> GB) then begin
			lua_pushlightuserdata( Mek^.QRL , GB );
		end else begin
			lua_pushnil( Mek^.QRL );
		end;
		DEBUG_NowInTheQRL := True;
		ret := lua_pcall( Mek^.QRL , 2 , 1 , 0 );
		DEBUG_NowInTheQRL := False;
		sysutils.StrDispose (pmsg);
		if ( 0 <> ret ) then begin
			if HeadMatchesString( QRL_Subroutine_Label , QRL_Ptr ) then begin
				s := Copy( QRL_Ptr , 1 + Length( QRL_Subroutine_Label ) , Length( QRL_Ptr ) );
				ErrorMessage_fork('ERROR: At Mek(' + GetMekName_for_Trace( Mek ) + '), Subroutine "' + s + '" was not found.' );
				DialogMsg('ERROR: At Mek(' + GetMekName_for_Trace( Mek ) + '), Subroutine "' + s + '" was not found.' );
				lua_pop( Mek^.QRL , 1 );
				exit (False);
			end;

			ErrorMessage_fork('ERROR: GetQRLInput( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_pcall(' + QRL_Ptr + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
			DialogMsg('ERROR: GetQRLInput( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_pcall(' + QRL_Ptr + ') failed : ' + lua_tostring( Mek^.QRL , -1 ) );
{$IFDEF DEBUG}
			QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
			lua_pop( Mek^.QRL , 1 );
			exit (False);
		end;

		if lua_isstring( Mek^.QRL , 1 ) then begin
			QRL_Ptr := lua_tostring( Mek^.QRL , 1 );
		end else begin
			ErrorMessage_fork('ERROR: GetQRLInput( Mek(' + GetMekName_for_Trace( Mek ) + ') ), lua_call(' + QRL_Ptr + ') result mismatch.' );
{$IFDEF DEBUG}
			QRL_PrintStack_glue( Mek^.QRL );
{$ENDIF DEBUG}
			QRL_Ptr := QRL_FunctionName_initialPoint;
		end;
		lua_pop( Mek^.QRL , 1 );
		SetSAtt( Mek^.SA , QRL_Ptr_TAG + ' <' + QRL_Ptr + '>' );

		QRL_ChipMode := SAttValueToInt( Mek^.SA , QRL_ChipMode_TAG );
		QRL_ChipMode_LoopCount := SAttValueToInt( Mek^.SA , QRL_ChipMode_LoopCount_TAG );

		if (QRL_ChipMode_EXEC = QRL_ChipMode) then begin
			break;
		end else if (QRL_ChipMode_BEGIN = QRL_ChipMode) then begin
			if (0 < QRL_ChipMode_LoopCount) then begin
				break;
			end;
		end else if (QRL_ChipMode_END = QRL_ChipMode) then begin
			continue;
		end;

		inc( QRL_ChipMode_LoopCount );
		SetSAtt( Mek^.SA , QRL_ChipMode_LoopCount_TAG + ' <' + BStr( QRL_ChipMode_LoopCount ) + '>' );
		if (QRL_ChipMode_LoopCount_Max <= QRL_ChipMode_LoopCount) then begin
			break;
		end;
	end;

{$IFDEF SDL2}
	if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
		QRL_Viewer_RefreshTracer();
	end;
{$ELSE SDL2}
{$ENDIF SDL2}

	GetQRLInput := True;
end;



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: qrl.pp');
{$ENDIF DEBUG}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: qrl.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
{$ENDIF PATCH_HEART}
