{$IFDEF PATCH_HEART}
unit qrl_glue;
{ Quasi Robot Language }

interface

uses sysutils,
{$IFDEF LUAJIT51}
	luajit51,
{$ELSE LUAJIT51}
	lua53,
{$ENDIF LUAJIT51}
	gears_base, gears;

Function GetMekName_for_Trace( Mek: GearPtr ): String;
Function QRL_PrintStack_glue( L: Plua_State ): LongInt; cdecl;
Function QRL_attach_glue( QRL: PLua_State ): Boolean;



implementation

uses errmsg, texutil,
	ui4gh, ability, action, aibrain, arenacfe, backpack, effects, damage, gearutil, ghchars, ghmodule, ghweapon, interact, locale, movement, pcaction, skilluse, rpgdice,
{$IFDEF SDLMODE}
	sdlgfx, sdlmap,
{$ELSE SDLMODE}
	congfx,context, conmap,
{$ENDIF SDLMODE}
	qrl_init
{$IFDEF SDL2}
	,qrl_viewer
{$ELSE SDL2}
{$ENDIF SDL2}
	;


{$IFDEF LUAJIT51}
procedure lua_pushstring(L: Plua_State; const s: AnsiString); inline;
begin
	lua_pushlstring(L, PAnsiChar(s), Length(s));
end;

function lua_isinteger(L: Plua_State; idx: Integer): LongBool; inline;
begin
	lua_isinteger := lua_isnumber( L , idx );
end;
{$ENDIF LUAJIT51}



Function GetMekName_for_Trace( Mek: GearPtr ): String;
begin
	GetMekName_for_Trace := FullGearName( LocatePilot( Mek ) ) + ':' + FullGearName( Mek );
end;


Function QRL_PrintStack_glue( L: Plua_State ): LongInt; cdecl;
	Procedure PrintStackItem( idx: Integer );
	var
		t: Integer;
	begin
		t := lua_type( L , idx );
		case t of
		LUA_TNIL:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + ';');
		LUA_TBOOLEAN:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_toboolean(L,idx)) + '"');
		LUA_TLIGHTUSERDATA:	ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_topointer(L,idx)) + '"');
		LUA_TNUMBER:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_tonumber(L,idx)) + '"');
		LUA_TSTRING:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + lua_tostring(L,idx) + '"');
		LUA_TTABLE:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + ';');
		LUA_TFUNCTION:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_topointer(L,idx)) + '"');
		LUA_TUSERDATA:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_touserdata(L,idx)) + '"');
		LUA_TTHREAD:		ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + '; "' + BStr(lua_topointer(L,idx)) + '"');
		else			ErrorMessage_fork('INFO: index ' + BStr(idx) + '; type ' + lua_typename(L,t) + ';');
		end;
	end;
var
	top: Integer;
	idx: Integer;
begin
	if ( NIL = L ) then begin
		ErrorMessage_fork('INFO: --- QRL_PrintStack_glue(NIL) ---');
		exit(0);
	end;
	ErrorMessage_fork('INFO: --- QRL_PrintStack_glue() stack dump begin ---');

	top := lua_gettop( L );
	for idx := top downto 1 do begin
		PrintStackItem( idx );
	end;

	ErrorMessage_fork('INFO: --- QRL_PrintStack_glue() stack dump end ---');
	QRL_PrintStack_glue := 0;
end;


Function PrintTraceMessage_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	msg: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: PrintTraceMessage_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	msg := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	if (DEBUG_TraceQRL and (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team ))) then begin
{$IFDEF DEBUG}
		ErrorMessage_fork('TRACE: ' + TimeString( GB^.ComTime ) + ' Mek(' + GetMekName_for_Trace( Mek ) + ') ' + msg + '' );
{$ENDIF DEBUG}
		DialogMsg('TRACE: ' + TimeString( GB^.ComTime ) + ' Mek(' + GetMekName_for_Trace( Mek ) + ') ' + msg + '' );
	end;

	PrintTraceMessage_glue := 0;
end;


Function PrintTracer_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	msg: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: PrintTracer_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	msg := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	if (DEBUG_TraceQRL and (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team ))) then begin
{$IFDEF SDL2}
		QRL_Viewer_ShowTracer( msg );
{$ELSE SDL2}
{$ENDIF SDL2}
	end;

	PrintTracer_glue := 0;
end;


Function PrintErrorMessage_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	msg: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: PrintErrorMessage_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	msg := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	if (NAV_DefPlayerTeam = NAttValue( Mek^.NA , NAG_Location , NAS_Team )) then begin
{$IFDEF DEBUG}
		ErrorMessage_fork('ERROR: ' + TimeString( GB^.ComTime ) + ' Mek(' + GetMekName_for_Trace( Mek ) + ') ' + msg + '' );
{$ENDIF DEBUG}
		DialogMsg('ERROR: ' + TimeString( GB^.ComTime ) + ' Mek(' + GetMekName_for_Trace( Mek ) + ') ' + msg + '' );
	end;

	PrintErrorMessage_glue := 0;
end;

Function GetMekName_for_Trace_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GetMekName_for_Trace_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_str := GetMekName_for_Trace( Mek );

	lua_pushstring( L , ret_str );
	GetMekName_for_Trace_glue := 1;
end;

Function GetSuspendFileName_glue( L: Plua_State ): LongInt; cdecl;
	Function LocatePC( GB: GameBoardPtr ): GearPtr;
	var
		Bits,PC: GearPtr;
	begin
		PC := NIL;
		Bits := GB^.Meks;
		while ( Bits <> NIL ) and ( PC = NIL ) do begin
			if (GG_DisposeGear < Bits^.G) then begin
				if ( NAttValue( Bits^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam ) and IsMasterGear( Bits ) and OnTheMap( Bits ) and GearOperational( Bits ) then begin
					PC := Bits;
				end;
			end;
			Bits := Bits^.Next;
		end;
		LocatePC := PC;
	end;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Pilot: String;
	BaseFileName: String;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: GetSuspendFileName_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	Pilot := SAttValue( Mek^.SA , QRL_Pilot_TAG );
	if Length(Pilot) <= 0 then begin
		Pilot := SAttValue( Mek^.SA , 'PILOT' );
	end;
	if Length(Pilot) <= 0 then begin
		Pilot := GearName( LocatePilot( Mek ) );
	end;
	if Length(Pilot) <= 0 then begin
		ret_str := '';
	end else begin
		BaseFileName := GB^.CampaignFileName;
		if ( Length( BaseFileName ) < 1 ) then begin
			BaseFileName := GB^.CampaignName;
		end;
		if GB^.ArenaMode then begin
			ret_str := Save_Unit_Base + TextEncode( BaseFileName ) + '.' + TextEncode( Pilot ) + QRL_SuspendFileExtName;
		end else begin
			ret_str := Save_Campaign_Base + TextEncode( BaseFileName ) + '.' + TextEncode( Pilot ) + QRL_SuspendFileExtName;
		end;
	end;

	lua_pushstring( L , ret_str );
	GetSuspendFileName_glue := 1;
end;


Function SetChipMode_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Mode: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: SetChipMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Mode := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	SetSAtt( Mek^.SA , QRL_ChipMode_TAG + ' <' + BStr( Mode ) + '>' );

	ret := ( 0 < SAttValueToInt( Mek^.SA , QRL_ChipMode_LoopCount_TAG ) );

	lua_pushboolean( L , ret );
	SetChipMode_glue := 1;
end;


Function RetrieveAString_glue( L: Plua_State ): LongInt; cdecl;
var
	in_str: String;
	ret_str: String;
begin
	if not lua_isstring( L , 1 ) then begin
		ErrorMessage_fork('ERROR: RetrieveAString_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	in_str := lua_tostring( L , 1 );
	lua_pop( L , 1 );

	ret_str := RetrieveAString( in_str );

	lua_pushstring( L , ret_str );
	RetrieveAString_glue := 1;
end;


Function GB_ComTime_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GB_ComTime_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GB^.ComTime;

	lua_pushinteger( L , ret_val );
	GB_ComTime_glue := 1;
end;

Function GB_Scale_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GB_Scale_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GB^.Scale;

	lua_pushinteger( L , ret_val );
	GB_Scale_glue := 1;
end;

Function GB_Map_Terr_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	X, Y: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: GB_Map_Terr_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	X := lua_tointeger( L , 2 );
	Y := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret_val := GB^.Map[ X , Y ].Terr;

	lua_pushinteger( L , ret_val );
	GB_Map_Terr_glue := 1;
end;

Function GB_Scene_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GB_Scene_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GB^.Scene;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	GB_Scene_glue := 1;
end;

Function GB_Meks_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GB_Meks_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GB^.Meks;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	GB_Meks_glue := 1;
end;

Function TerrMan_Altitude_glue( L: Plua_State ): LongInt; cdecl;
var
	Terr: Integer;
	ret_val: SmallInt;
begin
	if not lua_isinteger( L , 1 ) then begin
		ErrorMessage_fork('ERROR: TerrMan_Altitude_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Terr := lua_tointeger( L , 1 );
	lua_pop( L , 1 );

	ret_val := TerrMan[ Terr ].Altitude;

	lua_pushinteger( L , ret_val );
	TerrMan_Altitude_glue := 1;
end;

Function TerrMan_Obscurement_glue( L: Plua_State ): LongInt; cdecl;
var
	Terr: Integer;
	ret_val: Byte;
begin
	if not lua_isinteger( L , 1 ) then begin
		ErrorMessage_fork('ERROR: TerrMan_Obscurement_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Terr := lua_tointeger( L , 1 );
	lua_pop( L , 1 );

	ret_val := TerrMan[ Terr ].Obscurement;

	lua_pushinteger( L , ret_val );
	TerrMan_Obscurement_glue := 1;
end;

Function Mek_G_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_G_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.G;

	lua_pushinteger( L , ret_val );
	Mek_G_glue := 1;
end;

Function Mek_S_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_S_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.S;

	lua_pushinteger( L , ret_val );
	Mek_S_glue := 1;
end;

Function Mek_V_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_V_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.V;

	lua_pushinteger( L , ret_val );
	Mek_V_glue := 1;
end;

Function Mek_Scale_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_Scale_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.Scale;

	lua_pushinteger( L , ret_val );
	Mek_Scale_glue := 1;
end;

Function Mek_Stat_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	N: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_Stat_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	N := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := Mek^.Stat[ N ];

	lua_pushinteger( L , ret_val );
	Mek_Stat_glue := 1;
end;

Function Mek_Name_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_Name_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_str := GearName( Mek );

	lua_pushstring( L , ret_str );
	Mek_Name_glue := 1;
end;

Function Mek_SubCom_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_SubCom_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.SubCom;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	Mek_SubCom_glue := 1;
end;

Function Mek_InvCom_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_InvCom_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.InvCom;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	Mek_InvCom_glue := 1;
end;

Function Mek_Next_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_Next_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.Next;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	Mek_Next_glue := 1;
end;

Function Mek_Parent_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_Parent_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := Mek^.Parent;

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	Mek_Parent_glue := 1;
end;

Function Mek_NAttValue_EpisodeData_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_EpisodeData_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_EpisodeData;
	S := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_EpisodeData_glue := 1;
end;

Function Mek_NAttValue_Location_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_Location_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_Location;
	S := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_Location_glue := 1;
end;

Function Mek_NAttValue_Action_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_Action_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_Action;
	S := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_Action_glue := 1;
end;

Function Mek_NAttValue_Personal_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_Personal_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_Personal;
	S := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_Personal_glue := 1;
end;

Function Mek_NAttValue_Skill_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_Skill_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_Skill;
	S := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_Skill_glue := 1;
end;

Function Mek_NAttValue_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: Mek_NAttValue_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := lua_tointeger( L , 2 );
	S := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	Mek_NAttValue_glue := 1;
end;

Function Mek_SetNAtt_EpisodeData_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	V: LongInt;
	ret_ptr: NAttPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: Mek_SetNAtt_EpisodeData_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_EpisodeData;
	S := lua_tointeger( L , 2 );
	V := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret_ptr := SetNAtt( Mek^.NA , G , S , V );

	if (NIL <> ret_ptr) then begin
		lua_pushlightuserdata( L , ret_ptr );
	end else begin
		lua_pushnil( L );
	end;
	Mek_SetNAtt_EpisodeData_glue := 1;
end;

Function Mek_SetNAtt_Action_MoveMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	V: LongInt;
	ret_ptr: NAttPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_SetNAtt_Action_MoveMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_Action;
	S := NAS_MoveMode;
	V := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_ptr := SetNAtt( Mek^.NA , G , S , V );

	if (NIL <> ret_ptr) then begin
		lua_pushlightuserdata( L , ret_ptr );
	end else begin
		lua_pushnil( L );
	end;
	Mek_SetNAtt_Action_MoveMode_glue := 1;
end;

Function Mek_SetSAtt_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	Key: string;
	Value: string;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isstring( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: Mek_SetSAtt_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	Key := lua_tostring( L , 2 );
	Value := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	SetSAtt( Mek^.SA , ( Key + ' <' + Value + '>') );

	Mek_SetSAtt_glue := 0;
end;

Function Mek_SAttValue_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	Tag: string;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isstring( L , 2 ) then begin
		ErrorMessage_fork('ERROR: Mek_SAttValue_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	Tag := lua_tostring( L , 2 );
	lua_pop( L , 2 );

	ret_str := SAttValue( Mek^.SA , Tag );

	lua_pushstring( L , ret_str );
	Mek_SAttValue_glue := 1;
end;


Function Mek_GUID_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Mek_GUID_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	if ( NIL <> Mek ) then begin
		ret_str := GUIDToString( Mek^.GUID );
	end else begin
		ret_str := '';
	end;

	lua_pushstring( L , ret_str );
	Mek_GUID_glue := 1;
end;

Function GUID_to_Mek_glue( L: Plua_State ): LongInt; cdecl;
var
	GUID: TGUID;

	Function GUID_to_Mek( Mek: GearPtr ): GearPtr;
	var
		ret_Mek: GearPtr;
	begin
		while ( NIL <> Mek ) do begin
			if IsEqualGUID( GUID , Mek^.GUID ) then begin
				break;
			end;
			if ( NIL <> Mek^.SubCom ) then begin
				ret_Mek := GUID_to_Mek( Mek^.SubCom );
				if ( NIL <> ret_Mek ) then begin
					Mek := ret_Mek;
					break;
				end;
			end;
			if ( NIL <> Mek^.InvCom ) then begin
				ret_Mek := GUID_to_Mek( Mek^.InvCom );
				if ( NIL <> ret_Mek ) then begin
					Mek := ret_Mek;
					break;
				end;
			end;
			Mek := Mek^.Next;
		end;
		GUID_to_Mek := Mek;
	end;
var
	GB: GameBoardPtr;
	GUID_str: String;
	Mek: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isstring( L , 2 ) then begin
		ErrorMessage_fork('ERROR: GUID_to_Mek_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	GUID_str := lua_tostring( L , 2 );
	lua_pop( L , 2 );

	if ( 0 < Length(GUID_str) ) then begin
		GUID := StringToGUID( GUID_str );
		Mek := GUID_to_Mek( GB^.Meks );
		if ( NIL = Mek ) then begin
			ErrorMessage_fork('ERROR: GUID_to_Mek_glue(); GUID ' + GUID_str + ' was not found.');
		end;
	end else begin
		Mek := NIL;
		ErrorMessage_fork('ERROR: GUID_to_Mek_glue(); The format of the GUID is wrong.');
	end;

	lua_pushlightuserdata( L , Mek );
	GUID_to_Mek_glue := 1;
end;


Function IsMasterGear_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: IsMasterGear_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := IsMasterGear( Mek );

	lua_pushboolean( L , ret );
	IsMasterGear_glue := 1;
end;


Function FindRoot_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: FindRoot_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := FindRoot( Mek );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	FindRoot_glue := 1;
end;


Function IsSubCom_glue( L: Plua_State ): LongInt; cdecl;
var
	Part: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: IsSubCom_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Part := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := IsSubCom( Part );

	lua_pushboolean( L , ret );
	IsSubCom_glue := 1;
end;


Function IsInvCom_glue( L: Plua_State ): LongInt; cdecl;
var
	Part: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: IsInvCom_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Part := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := IsInvCom( Part );

	lua_pushboolean( L , ret );
	IsInvCom_glue := 1;
end;


Function CalcMoveTime_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	GB: GameBoardPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: CalcMoveTime_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	GB := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret_val := CalcMoveTime( Mek , GB );

	lua_pushinteger( L , ret_val );
	CalcMoveTime_glue := 1;
end;

Function WeaponDC_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	AtOp: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: WeaponDC_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	AtOp := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := WeaponDC( Mek , AtOp );

	lua_pushinteger( L , ret_val );
	WeaponDC_glue := 1;
end;

Function GearCurrentDamage_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearCurrentDamage_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GearCurrentDamage( Mek );

	lua_pushinteger( L , ret_val );
	GearCurrentDamage_glue := 1;
end;

Function GearMaxDamage_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearMaxDamage_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GearMaxDamage( Mek );

	lua_pushinteger( L , ret_val );
	GearMaxDamage_glue := 1;
end;

Function GearCurrentArmor_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearCurrentArmor_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GearCurrentArmor( Mek );

	lua_pushinteger( L , ret_val );
	GearCurrentArmor_glue := 1;
end;

Function GearMaxArmor_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearMaxArmor_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := GearMaxArmor( Mek );

	lua_pushinteger( L , ret_val );
	GearMaxArmor_glue := 1;
end;

Function PercentDamaged_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: PercentDamaged_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := PercentDamaged( Mek );

	lua_pushinteger( L , ret_val );
	PercentDamaged_glue := 1;
end;

Function NotDestroyed_glue( L: Plua_State ): LongInt; cdecl;
var
	Part: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: NotDestroyed_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Part := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := NotDestroyed( Part );

	lua_pushboolean( L , ret );
	NotDestroyed_glue := 1;
end;

Function Destroyed_glue( L: Plua_State ): LongInt; cdecl;
var
	Part: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Destroyed_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Part := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := Destroyed( Part );

	lua_pushboolean( L , ret );
	Destroyed_glue := 1;
end;

Function TotalRepairableDamage_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	Skill: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: TotalRepairableDamage_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	Skill := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := TotalRepairableDamage( Mek , Skill );

	lua_pushinteger( L , ret_val );
	TotalRepairableDamage_glue := 1;
end;

Function AreEnemies_TT_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	T1: Integer;
	T2: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: AreEnemies_TT_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	T1 := lua_tointeger( L , 2 );
	T2 := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret := AreEnemies( GB , T1 , T2 );

	lua_pushboolean( L , ret );
	AreEnemies_TT_glue := 1;
end;

Function AreEnemies_MM_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	M1: GearPtr;
	M2: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: AreEnemies_MM_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	M1 := lua_topointer( L , 2 );
	M2 := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret := AreEnemies( GB , M1 , M2 );

	lua_pushboolean( L , ret );
	AreEnemies_MM_glue := 1;
end;

Function TeamSkill_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	V: Integer;
	S: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: TeamSkill_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	V := lua_tointeger( L , 2 );
	S := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret_val := TeamSkill( GB , V , S );

	lua_pushinteger( L , ret_val );
	TeamSkill_glue := 1;
end;

Function BaseMoveRate_withMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	MM: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: BaseMoveRate_withMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	MM := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := BaseMoveRate( Mek , MM );

	lua_pushinteger( L , ret_val );
	BaseMoveRate_withMode_glue := 1;
end;

Function BaseMoveRate_CurrentMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: BaseMoveRate_CurrentMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := BaseMoveRate( Mek );

	lua_pushinteger( L , ret_val );
	BaseMoveRate_CurrentMode_glue := 1;
end;

Function SkillValue_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	v: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: SkillValue_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	v := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := SkillValue( Mek , v );

	lua_pushinteger( L , ret_val );
	SkillValue_glue := 1;
end;

Function AreAllies_TT_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	T1: Integer;
	T2: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: AreAllies_TT_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	T1 := lua_tointeger( L , 2 );
	T2 := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret := AreAllies( GB , T1 , T2 );

	lua_pushboolean( L , ret );
	AreAllies_TT_glue := 1;
end;

Function AreAllies_MM_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	M1: GearPtr;
	M2: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: AreAllies_MM_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	M1 := lua_topointer( L , 2 );
	M2 := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret := AreAllies( GB , M1 , M2 );

	lua_pushboolean( L , ret );
	AreAllies_MM_glue := 1;
end;

Function IsSafeArea_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: IsSafeArea_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := IsSafeArea( GB );

	lua_pushboolean( L , ret );
	IsSafeArea_glue := 1;
end;

Function GearCurrentLocation_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	P: Point;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearCurrentLocation_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	P := GearCurrentLocation( Mek );

	lua_pushinteger( L , P.X );
	lua_pushinteger( L , P.Y );
	lua_pushinteger( L , P.Z );
	GearCurrentLocation_glue := 3;
end;

Function OnTheMap_XY_glue( L: Plua_State ): LongInt; cdecl;
var
	X: Integer;
	Y: Integer;
	ret: Boolean;
begin
	if not lua_isinteger( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: OnTheMap_XY_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	X := lua_tointeger( L , 1 );
	Y := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret := OnTheMap( X , Y );

	lua_pushboolean( L , ret );
	OnTheMap_XY_glue := 1;
end;

Function OnTheMap_Mek_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: OnTheMap_Mek_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := OnTheMap( Mek );

	lua_pushboolean( L , ret );
	OnTheMap_Mek_glue := 1;
end;

Function MekAltitude_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: MekAltitude_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret_val := MekAltitude( GB , Mek );

	lua_pushinteger( L , ret_val );
	MekAltitude_glue := 1;
end;

Function FindBlockerXYZ_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	X, Y, Z: Integer;
	ret_Mek: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) or not lua_isinteger( L , 4 ) then begin
		ErrorMessage_fork('ERROR: FindBlockerXYZ_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	X := lua_tointeger( L , 2 );
	Y := lua_tointeger( L , 3 );
	Z := lua_tointeger( L , 4 );
	lua_pop( L , 4 );

	ret_Mek := FindBlockerXYZ( GB , X , Y , Z );

	if (NIL <> ret_Mek) then begin
		lua_pushlightuserdata( L , ret_Mek );
	end else begin
		lua_pushnil( L );
	end;
	FindBlockerXYZ_glue := 1;
end;

Function UpdateShadowMap_for_General_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: UpdateShadowMap_for_General_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	UpdateShadowMap( GB );

	UpdateShadowMap_for_General_glue := 0;
end;

Function TileBlocksLOS_for_General_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	X: Integer;
	Y: Integer;
	Z: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) or not lua_isinteger( L , 4 ) then begin
		ErrorMessage_fork('ERROR: TileBlocksLOS_for_General_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	X := lua_tointeger( L , 2 );
	Y := lua_tointeger( L , 3 );
	Z := lua_tointeger( L , 4 );
	lua_pop( L , 4 );

	ret := TileBlocksLOS( GB , X , Y , Z );

	lua_pushboolean( L , ret );
	TileBlocksLOS_for_General_glue := 1;
end;

Function GearDestination_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	P: Point;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearDestination_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	P := GearDestination( Mek );

	lua_pushinteger( L , P.X );
	lua_pushinteger( L , P.Y );
	lua_pushinteger( L , P.Z );
	GearDestination_glue := 3;
end;

Function IsObstacle_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Terrain: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: IsObstacle_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Terrain := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret := IsObstacle( GB , Mek , Terrain );

	lua_pushboolean( L , ret );
	IsObstacle_glue := 1;
end;

Function MoveBlocked_for_General_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	GB: GameBoardPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: MoveBlocked_for_General_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	GB := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret := MoveBlocked( Mek , GB );

	lua_pushboolean( L , ret );
	MoveBlocked_for_General_glue := 1;
end;

Function IsBlocked_for_General_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	GB: GameBoardPtr;
	X: Integer;
	Y: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) or not lua_isinteger( L , 4 ) then begin
		ErrorMessage_fork('ERROR: IsBlocked_for_General_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	GB := lua_topointer( L , 2 );
	X := lua_tointeger( L , 3 );
	Y := lua_tointeger( L , 4 );
	lua_pop( L , 4 );

	ret := IsBlocked( Mek , GB , X , Y );

	lua_pushboolean( L , ret );
	IsBlocked_for_General_glue := 1;
end;

Function IsInCover_for_General_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	GB: GameBoardPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: IsInCover_for_General_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret := IsInCover( GB , Mek );

	lua_pushboolean( L , ret );
	IsInCover_for_General_glue := 1;
end;

Function DoorPresent_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	X: Integer;
	Y: Integer;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: DoorPresent_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	X := lua_tointeger( L , 2 );
	Y := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret := DoorPresent( GB , X , Y );

	lua_pushboolean( L , ret );
	DoorPresent_glue := 1;
end;

Function JumpTime_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: JumpTime_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := JumpTime( Mek );

	lua_pushinteger( L , ret_val );
	JumpTime_glue := 1;
end;

Function MoveLegal_withMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	MoveMode: Integer;
	MoveAction: Integer;
	ComTime: LongInt;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) or not lua_isinteger( L , 4 ) then begin
		ErrorMessage_fork('ERROR: MoveLegal_withMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	MoveMode := lua_tointeger( L , 2 );
	MoveAction := lua_tointeger( L , 3 );
	ComTime := lua_tointeger( L , 4 );
	lua_pop( L , 4 );

	ret := MoveLegal( Mek , MoveMode , MoveAction , ComTime );

	lua_pushboolean( L , ret );
	MoveLegal_withMode_glue := 1;
end;

Function MoveLegal_CurrentMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	MoveAction: Integer;
	ComTime: LongInt;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: MoveLegal_CurrentMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	MoveAction := lua_tointeger( L , 2 );
	ComTime := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret := MoveLegal( Mek , MoveAction , ComTime );

	lua_pushboolean( L , ret );
	MoveLegal_CurrentMode_glue := 1;
end;

Function LocateTeam_GB_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Team: Integer;
	ret_TG: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: LocateTeam_GB_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Team := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_TG := LocateTeam( GB , Team );

	if (NIL <> ret_TG) then begin
		lua_pushlightuserdata( L , ret_TG );
	end else begin
		lua_pushnil( L );
	end;
	LocateTeam_GB_glue := 1;
end;

{$IFDEF PATCH_CHEAT}
Function CheckConversionSystem_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: CheckConversionSystem_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := CheckConversionSystem( Mek , False );

	lua_pushboolean( L , ret );
	CheckConversionSystem_glue := 1;
end;
{$ENDIF PATCH_CHEAT}

Function ReadyToFire_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	User: GearPtr;
	Weapon: GearPtr;
	CheckForm: Boolean;
	CheckTime: Boolean;
	CheckAmmo: Boolean;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_isboolean( L , 4 ) or not lua_isboolean( L , 5 ) or not lua_isboolean( L , 6 ) then begin
		ErrorMessage_fork('ERROR: ReadyToFire_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	User := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	CheckForm := lua_toboolean( L , 4 );
	CheckAmmo := lua_toboolean( L , 5 );
	CheckTime := lua_toboolean( L , 6 );
	lua_pop( L , 6 );

	ret := ReadyToFire( GB , User , Weapon , CheckForm , CheckAmmo , CheckTime );

	lua_pushboolean( L , ret );
	ReadyToFire_glue := 1;
end;

Function Range_MM_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	M1: GearPtr;
	M2: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: Range_MM_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	M1 := lua_topointer( L , 2 );
	M2 := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret_val := Range( GB , M1 , M2 );

	lua_pushinteger( L , ret_val );
	Range_MM_glue := 1;
end;

Function WeaponRange_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Weapon: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: WeaponRange_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Weapon := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret_val := WeaponRange( GB , Weapon );

	lua_pushinteger( L , ret_val );
	WeaponRange_glue := 1;
end;

Function RangeCheck_Mek_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Master: GearPtr;
	Weapon: GearPtr;
	Target: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) then begin
		ErrorMessage_fork('ERROR: RangeCheck_Mek_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Master := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Target := lua_topointer( L , 4 );
	lua_pop( L , 4 );

	ret := RangeCheck( GB , Master , Weapon , Target );

	lua_pushboolean( L , ret );
	RangeCheck_Mek_glue := 1;
end;

Function RangeArcCheck_Mek_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Master: GearPtr;
	Weapon: GearPtr;
	Target: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) then begin
		ErrorMessage_fork('ERROR: RangeArcCheck_Mek_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Master := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Target := lua_topointer( L , 4 );
	lua_pop( L , 4 );

	ret := RangeArcCheck( GB , Master , Weapon , Target );

	lua_pushboolean( L , ret );
	RangeArcCheck_Mek_glue := 1;
end;

Function LocatePilot_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: LocatePilot_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := LocatePilot( Mek );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	LocatePilot_glue := 1;
end;

Function GearOperational_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearOperational_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := GearOperational( Mek );

	lua_pushboolean( L , ret );
	GearOperational_glue := 1;
end;

Function GearActive_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearActive_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := GearActive( Mek );

	lua_pushboolean( L , ret );
	GearActive_glue := 1;
end;

Function TeamCanSeeTarget_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Team: Integer;
	Target: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: TeamCanSeeTarget_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Team := lua_tointeger( L , 2 );
	Target := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret := TeamCanSeeTarget( GB , Team , Target );

	lua_pushboolean( L , ret );
	TeamCanSeeTarget_glue := 1;
end;

Function MekCanSeeTarget_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Target: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: MekCanSeeTarget_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Target := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret := MekCanSeeTarget( GB , Mek , Target );

	lua_pushboolean( L , ret );
	MekCanSeeTarget_glue := 1;
end;

Function LocateMekByUID_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	UID: Integer;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: LocateMekByUID_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	UID := lua_tointeger( L , 2 );
	lua_pop( L , 2 );

	ret_val := LocateMekByUID( GB , UID );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	LocateMekByUID_glue := 1;
end;

Function SeekGear_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: SeekGear_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := lua_tointeger( L , 2 );
	S := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	ret_val := SeekGear( Mek , G , S );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	SeekGear_glue := 1;
end;

Function SeekGearByName_glue( L: Plua_State ): LongInt; cdecl;
var
	Meks: GearPtr;
	Name: String;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isstring( L , 2 ) then begin
		ErrorMessage_fork('ERROR: SeekGearByName_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Meks := lua_topointer( L , 1 );
	Name := lua_tostring( L , 2 );
	lua_pop( L , 2 );

	ret_val := SeekGearByName( Meks , Name );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	SeekGearByName_glue := 1;
end;

Function SeekGearByPilotName_glue( L: Plua_State ): LongInt; cdecl;
	function SeekGearByPilotName( LList: GearPtr; Name: String ): GearPtr;
	var
		it: GearPtr;
		p_n: String;
	begin
		it := NIL;
		Name := UpCase( Name );
		while ( NIL <> LList ) do begin
{$IFDEF PATCH_GH}
			if (GG_DisposeGear < LList^.G) then begin
{$ENDIF PATCH_GH}
			p_n := UpCase( GearName( LocatePilot( LList ) ) );
			if ( p_n = Name ) then it := LList;
{$IFDEF PATCH_GH}
			end;
{$ENDIF PATCH_GH}
			LList := LList^.Next;
		end;
		SeekGearByPilotName := it;
	end;
var
	Meks: GearPtr;
	Name: String;
	ret_val: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isstring( L , 2 ) then begin
		ErrorMessage_fork('ERROR: SeekGearByPilotName_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Meks := lua_topointer( L , 1 );
	Name := lua_tostring( L , 2 );
	lua_pop( L , 2 );

	ret_val := SeekGearByPilotName( Meks , Name );

	if (NIL <> ret_val) then begin
		lua_pushlightuserdata( L , ret_val );
	end else begin
		lua_pushnil( L );
	end;
	SeekGearByPilotName_glue := 1;
end;

Function CharCurrentMental_glue( L: Plua_State ): LongInt; cdecl;
var
	PC: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: CharCurrentMental_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	PC := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := CharCurrentMental( PC );

	lua_pushinteger( L , ret_val );
	CharCurrentMental_glue := 1;
end;

Function ReactionTime_glue( L: Plua_State ): LongInt; cdecl;
var
	Master: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: ReactionTime_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Master := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := ReactionTime( Master );

	lua_pushinteger( L , ret_val );
	ReactionTime_glue := 1;
end;

Function IsSexy_glue( L: Plua_State ): LongInt; cdecl;
var
	PC: GearPtr;
	NPC: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: IsSexy_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	PC := lua_topointer( L , 1 );
	NPC := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret := IsSexy( PC , NPC );

	lua_pushboolean( L , ret );
	IsSexy_glue := 1;
end;


Function TeamOrders_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	N: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: TeamOrders_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	N := STAT_TeamOrders;
	lua_pop( L , 1 );

	ret_val := Mek^.Stat[ N ];

	lua_pushinteger( L , ret_val );
	TeamOrders_glue := 1;
end;

Function AmmoPresent_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	N: Integer;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: AmmoPresent_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	N := STAT_AmmoPresent;
	lua_pop( L , 1 );

	ret_val := Mek^.Stat[ N ];

	lua_pushinteger( L , ret_val );
	AmmoPresent_glue := 1;
end;

Function AmmoSpent_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	G: Integer;
	S: Integer;
	ret_val: LongInt;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: AmmoSpent_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	G := NAG_WeaponModifier;
	S := NAS_AmmoSpent;
	lua_pop( L , 1 );

	ret_val := NAttValue( Mek^.NA , G , S );

	lua_pushinteger( L , ret_val );
	AmmoSpent_glue := 1;
end;

Function NeedAmmo_glue( L: Plua_State ): LongInt; cdecl;
var
	Weapon: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: NeedAmmo_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Weapon := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret := NeedAmmo( Weapon );

	lua_pushboolean( L , ret );
	NeedAmmo_glue := 1;
end;

Function GoodAmmo_glue( L: Plua_State ): LongInt; cdecl;
var
	Weapon: GearPtr;
	Ammo: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: GoodAmmo_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Weapon := lua_topointer( L , 1 );
	Ammo := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	ret := not NotGoodAmmo( Weapon , Ammo );

	lua_pushboolean( L , ret );
	GoodAmmo_glue := 1;
end;

Function InstallAmmo_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Weapon: GearPtr;
	Ammo: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) then begin
		ErrorMessage_fork('ERROR: InstallAmmo_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Ammo := lua_topointer( L , 4 );
	lua_pop( L , 4 );

	ret := InstallAmmo( GB , Mek , Weapon , Ammo );

	lua_pushboolean( L , ret );
	InstallAmmo_glue := 1;
end;

Function ReloadAmmo_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Weapon: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: ReloadAmmo_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret := ReloadAmmo( GB , Mek , Weapon );

	lua_pushboolean( L , ret );
	ReloadAmmo_glue := 1;
end;

Function Remaining_of_Ammo_glue( L: Plua_State ): LongInt; cdecl;
var
	Ammo: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Remaining_of_Ammo_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Ammo := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	ret_val := 0;
	if ( NIL <> Ammo ) then begin
		ret_val := Ammo^.Stat[STAT_AmmoPresent] - NAttValue( Ammo^.NA , NAG_WeaponModifier , NAS_AmmoSpent );
	end;

	lua_pushinteger( L , ret_val );
	Remaining_of_Ammo_glue := 1;
end;


Function MaxSeparableMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: MaxSeparableMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_SEPARABLE;
	lua_pop( L , 1 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	MaxSeparableMode_glue := 1;
end;

Function IsSeparable_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_str: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: IsSeparable_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_SEPARATE + BStr( lua_tointeger( L , 2 ) );
	lua_pop( L , 2 );

	ret_str := SAttValue( Mek^.SA , tag );

	lua_pushstring( L , ret_str );
	IsSeparable_glue := 1;
end;

Function SeparateWait_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: SeparateWait_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_SEPARATE_WAIT + BStr( lua_tointeger( L , 2 ) );
	lua_pop( L , 2 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	SeparateWait_glue := 1;
end;


Function MaxTransformableMode_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: MaxTransformableMode_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_TRANSFORMABLE;
	lua_pop( L , 1 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	MaxTransformableMode_glue := 1;
end;

Function Transform_CurrentForm_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: Transform_CurrentForm_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_TRANSFORM_CURRENT;
	lua_pop( L , 1 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	Transform_CurrentForm_glue := 1;
end;

Function TransformWait_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: TransformWait_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_TRANSFORM_WAIT + BStr( lua_tointeger( L , 2 ) ) + '_' + BStr( lua_tointeger( L , 3 ) );
	lua_pop( L , 3 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	TransformWait_glue := 1;
end;

Function TransformGS_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	tag: string;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_isinteger( L , 2 ) then begin
		ErrorMessage_fork('ERROR: TransformGS_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	tag := SATT_TRANSFORM_GS + BStr( lua_tointeger( L , 2 ) );
	lua_pop( L , 2 );

	ret_val := SAttValueToInt( Mek^.SA , tag );

	lua_pushinteger( L , ret_val );
	TransformGS_glue := 1;
end;


Function WaitAMinute_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	D: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: WaitAMinute_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	D := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	WaitAMinute( GB , Mek , D );

	WaitAMinute_glue := 0;
end;

Function PrepAction_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	V: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: PrepAction_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	V := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	PrepAction( GB , Mek , V );

	PrepAction_glue := 0;
end;

Function GearUp_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) then begin
		ErrorMessage_fork('ERROR: GearUp_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	lua_pop( L , 1 );

	GearUp( Mek );

	GearUp_glue := 0;
end;

Function RollStep_glue( L: Plua_State ): LongInt; cdecl;
var
	r: Integer;
	ret_val: Integer;
begin
	if not lua_isinteger( L , 1 ) then begin
		ErrorMessage_fork('ERROR: RollStep_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	r := lua_tointeger( L , 1 );
	lua_pop( L , 1 );

	ret_val := RollStep( r );

	lua_pushinteger( L , ret_val );
	RollStep_glue := 1;
end;

Function AIRepair_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	NPC: GearPtr;
	Target: GearPtr;
	RepairLimit: LongInt;
	Skill: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_isinteger( L , 4 ) or not lua_isinteger( L , 5 ) then begin
		ErrorMessage_fork('ERROR: AIRepair_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	NPC := lua_topointer( L , 2 );
	Target := lua_topointer( L , 3 );
	RepairLimit := lua_tointeger( L , 4 );
	Skill := lua_tointeger( L , 5 );
	lua_pop( L , 5 );

	AIRepair( GB , NPC , Target , RepairLimit , Skill );

	AIRepair_glue := 0;
end;

Function NPC_Flirtation_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	NPC: GearPtr;
	Target: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: NPC_Flirtation_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	NPC := lua_topointer( L , 2 );
	Target := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	NPC_Flirtation( GB , NPC , Target );

	NPC_Flirtation_glue := 0;
end;

Function NPC_Chatting_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	NPC: GearPtr;
	Target: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: NPC_Chatting_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	NPC := lua_topointer( L , 2 );
	Target := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	NPC_Chatting( GB , NPC , Target );

	NPC_Chatting_glue := 0;
end;

Function AI_Eject_glue( L: Plua_State ): LongInt; cdecl;
var
	Mek: GearPtr;
	GB: GameBoardPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) then begin
		ErrorMessage_fork('ERROR: AI_Eject_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	Mek := lua_topointer( L , 1 );
	GB := lua_topointer( L , 2 );
	lua_pop( L , 2 );

	AI_Eject( Mek , GB );

	AI_Eject_glue := 0;
end;

Function SafeToFire_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Weapon: GearPtr;
	Target: GearPtr;
	ret: Boolean;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) then begin
		ErrorMessage_fork('ERROR: SafeToFire_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Target := lua_topointer( L , 4 );
	lua_pop( L , 4 );

	ret := SafeToFire( GB , Mek , Weapon , Target );

	lua_pushboolean( L , ret );
	SafeToFire_glue := 1;
end;

Function WGoodness_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Weapon: GearPtr;
	Target: GearPtr;
	ret_val: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) then begin
		ErrorMessage_fork('ERROR: WGoodness_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Weapon := lua_topointer( L , 2 );
	Target := lua_topointer( L , 3 );
	lua_pop( L , 3 );

	ret_val := WGoodness( GB , Weapon , Target );

	lua_pushinteger( L , ret_val );
	WGoodness_glue := 1;
end;

Function AIAttacker_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Weapon: GearPtr;
	Target: GearPtr;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) then begin
		ErrorMessage_fork('ERROR: AIAttacker_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Target := lua_topointer( L , 4 );
	lua_pop( L , 4 );

	AIAttacker( GB , Mek , Weapon , Target );

	AIAttacker_glue := 0;
end;

Function AttackerFrontEnd_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	Weapon: GearPtr;
	Target: GearPtr;
	AtOp: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_islightuserdata( L , 3 ) or not lua_islightuserdata( L , 4 ) or not lua_isinteger( L , 5 ) then begin
		ErrorMessage_fork('ERROR: AttackerFrontEnd_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	Weapon := lua_topointer( L , 3 );
	Target := lua_topointer( L , 4 );
	AtOp := lua_tointeger( L , 5 );
	lua_pop( L , 5 );

	AttackerFrontEnd( GB , Mek , Weapon , Target , AtOp );

	AttackerFrontEnd_glue := 0;
end;

{$IFDEF PATCH_CHEAT}
Function DoTransformation_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	NewForm: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: DoTransformation_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	NewForm := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	DoTransformation( GB , Mek , NewForm );

	DoTransformation_glue := 0;
end;

Function DoPurgeParts_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	SelectMode: Integer;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isinteger( L , 3 ) then begin
		ErrorMessage_fork('ERROR: DoPurgeParts_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	SelectMode := lua_tointeger( L , 3 );
	lua_pop( L , 3 );

	DoPurgeParts( GB , Mek , SelectMode );

	DoPurgeParts_glue := 0;
end;
{$ENDIF PATCH_CHEAT}


Function NPC_CombatTaunt_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	msg: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: NPC_CombatTaunt_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	msg := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	NPC_CombatTaunt( GB, Mek, msg );

	NPC_CombatTaunt_glue := 0;
end;


Function NPC_Speaks_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	msg: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: NPC_Speaks_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	msg := lua_tostring( L , 3 );
	lua_pop( L , 3 );

	DialogMsg( '[' + GearName( LocatePilot( Mek ) ) + ']: ' + msg );

	NPC_Speaks_glue := 0;
end;


Function SoundEffect_glue( L: Plua_State ): LongInt; cdecl;
var
	GB: GameBoardPtr;
	Mek: GearPtr;
	name: String;
begin
	if not lua_islightuserdata( L , 1 ) or not lua_islightuserdata( L , 2 ) or not lua_isstring( L , 3 ) then begin
		ErrorMessage_fork('ERROR: SoundEffect_glue(); Illigal argument.');
{$IFDEF DEBUG}
		QRL_PrintStack_glue( L );
{$ENDIF DEBUG}
		exit (0);
	end;
	GB := lua_topointer( L , 1 );
	Mek := lua_topointer( L , 2 );
	name := lua_tostring( L , 3 );
	lua_pop( L , 3 );

{$IFDEF PATCH_AUDIO}
	PlaySoundEffect_Direct( name );
{$ENDIF PATCH_AUDIO}

	SoundEffect_glue := 0;
end;


Function QRL_attach_glue( QRL: PLua_State ): Boolean;
begin
	if ( NIL = QRL ) then begin
		exit (False);
	end;

	lua_register( QRL , 'PrintStack' , @QRL_PrintStack_glue );
	lua_register( QRL , 'PrintTraceMessage' , @PrintTraceMessage_glue );
	lua_register( QRL , 'PrintTracer' , @PrintTracer_glue );
	lua_register( QRL , 'PrintErrorMessage' , @PrintErrorMessage_glue );
	lua_register( QRL , 'GetMekName_for_Trace' , @GetMekName_for_Trace_glue );
	lua_register( QRL , 'GetSuspendFileName' , @GetSuspendFileName_glue );

	lua_register( QRL , 'SetChipMode' , @SetChipMode_glue );

	lua_register( QRL , 'RetrieveAString' , @RetrieveAString_glue );

	lua_register( QRL , 'GB_ComTime' , @GB_ComTime_glue );
	lua_register( QRL , 'GB_Scale' , @GB_Scale_glue );
	lua_register( QRL , 'GB_Map_Terr' , @GB_Map_Terr_glue );
	lua_register( QRL , 'GB_Scene' , @GB_Scene_glue );
	lua_register( QRL , 'GB_Meks' , @GB_Meks_glue );
	lua_register( QRL , 'TerrMan_Altitude' , @TerrMan_Altitude_glue );
	lua_register( QRL , 'TerrMan_Obscurement' , @TerrMan_Obscurement_glue );
	lua_register( QRL , 'Mek_G' , @Mek_G_glue );
	lua_register( QRL , 'Mek_S' , @Mek_S_glue );
	lua_register( QRL , 'Mek_V' , @Mek_V_glue );
	lua_register( QRL , 'Mek_Scale' , @Mek_Scale_glue );
	lua_register( QRL , 'Mek_Stat' , @Mek_Stat_glue );
	lua_register( QRL , 'Mek_Name' , @Mek_Name_glue );
	lua_register( QRL , 'Mek_SubCom' , @Mek_SubCom_glue );
	lua_register( QRL , 'Mek_InvCom' , @Mek_InvCom_glue );
	lua_register( QRL , 'Mek_Next' , @Mek_Next_glue );
	lua_register( QRL , 'Mek_Parent' , @Mek_Parent_glue );
	lua_register( QRL , 'Mek_NAttValue_EpisodeData' , @Mek_NAttValue_EpisodeData_glue );
	lua_register( QRL , 'Mek_NAttValue_Location' , @Mek_NAttValue_Location_glue );
	lua_register( QRL , 'Mek_NAttValue_Action' , @Mek_NAttValue_Action_glue );
	lua_register( QRL , 'Mek_NAttValue_Personal' , @Mek_NAttValue_Personal_glue );
	lua_register( QRL , 'Mek_NAttValue_Skill' , @Mek_NAttValue_Skill_glue );
	lua_register( QRL , 'Mek_NAttValue' , @Mek_NAttValue_glue );
	lua_register( QRL , 'Mek_SetNAtt_EpisodeData' , @Mek_SetNAtt_EpisodeData_glue );
	lua_register( QRL , 'Mek_SetNAtt_Action_MoveMode' , @Mek_SetNAtt_Action_MoveMode_glue );
	lua_register( QRL , 'Mek_SetSAtt' , @Mek_SetSAtt_glue );
	lua_register( QRL , 'Mek_SAttValue' , @Mek_SAttValue_glue );

	lua_register( QRL , 'Mek_GUID' , @Mek_GUID_glue );
	lua_register( QRL , 'GUID_to_Mek' , @GUID_to_Mek_glue );

	lua_register( QRL , 'IsMasterGear' , @IsMasterGear_glue );

	lua_register( QRL , 'FindRoot' , @FindRoot_glue );
	lua_register( QRL , 'IsSubCom' , @IsSubCom_glue );
	lua_register( QRL , 'IsInvCom' , @IsInvCom_glue );

	lua_register( QRL , 'CalcMoveTime' , @CalcMoveTime_glue );
	lua_register( QRL , 'WeaponDC' , @WeaponDC_glue );
	lua_register( QRL , 'GearCurrentDamage' , @GearCurrentDamage_glue );
	lua_register( QRL , 'GearMaxDamage' , @GearMaxDamage_glue );
	lua_register( QRL , 'GearCurrentArmor' , @GearCurrentArmor_glue );
	lua_register( QRL , 'GearMaxArmor' , @GearMaxArmor_glue );
	lua_register( QRL , 'PercentDamaged' , @PercentDamaged_glue );
	lua_register( QRL , 'NotDestroyed' , @NotDestroyed_glue );
	lua_register( QRL , 'Destroyed' , @Destroyed_glue );
	lua_register( QRL , 'TotalRepairableDamage' , @TotalRepairableDamage_glue );
	lua_register( QRL , 'AreEnemies_TT' , @AreEnemies_TT_glue );
	lua_register( QRL , 'AreEnemies_MM' , @AreEnemies_MM_glue );
	lua_register( QRL , 'TeamSkill' , @TeamSkill_glue );
	lua_register( QRL , 'BaseMoveRate_withMode' , @BaseMoveRate_withMode_glue );
	lua_register( QRL , 'BaseMoveRate_CurrentMode' , @BaseMoveRate_CurrentMode_glue );
	lua_register( QRL , 'SkillValue' , @SkillValue_glue );

	lua_register( QRL , 'AreAllies_TT' , @AreAllies_TT_glue );
	lua_register( QRL , 'AreAllies_MM' , @AreAllies_MM_glue );
	lua_register( QRL , 'IsSafeArea' , @IsSafeArea_glue );
	lua_register( QRL , 'GearCurrentLocation' , @GearCurrentLocation_glue );
	lua_register( QRL , 'OnTheMap_XY' , @OnTheMap_XY_glue );
	lua_register( QRL , 'OnTheMap_Mek' , @OnTheMap_Mek_glue );
	lua_register( QRL , 'MekAltitude' , @MekAltitude_glue );
	lua_register( QRL , 'FindBlockerXYZ' , @FindBlockerXYZ_glue );
	lua_register( QRL , 'UpdateShadowMap_for_General' , @UpdateShadowMap_for_General_glue );
	lua_register( QRL , 'TileBlocksLOS_for_General' , @TileBlocksLOS_for_General_glue );
	lua_register( QRL , 'GearDestination' , @GearDestination_glue );
	lua_register( QRL , 'IsObstacle' , @IsObstacle_glue );
	lua_register( QRL , 'MoveBlocked_for_General' , @MoveBlocked_for_General_glue );
	lua_register( QRL , 'IsBlocked_for_General' , @IsBlocked_for_General_glue );
	lua_register( QRL , 'IsInCover_for_General' , @IsInCover_for_General_glue );

	lua_register( QRL , 'DoorPresent' , @DoorPresent_glue );
	lua_register( QRL , 'JumpTime' , @JumpTime_glue );
	lua_register( QRL , 'MoveLegal_withMode' , @MoveLegal_withMode_glue );
	lua_register( QRL , 'MoveLegal_CurrentMode' , @MoveLegal_CurrentMode_glue );
	lua_register( QRL , 'LocateTeam_GB' , @LocateTeam_GB_glue );
{$IFDEF PATCH_CHEAT}
	lua_register( QRL , 'CheckConversionSystem' , @CheckConversionSystem_glue );
{$ENDIF PATCH_CHEAT}
	lua_register( QRL , 'ReadyToFire' , @ReadyToFire_glue );
	lua_register( QRL , 'Range_MM' , @Range_MM_glue );
	lua_register( QRL , 'WeaponRange' , @WeaponRange_glue );
	lua_register( QRL , 'RangeCheck_Mek' , @RangeCheck_Mek_glue );
	lua_register( QRL , 'RangeArcCheck_Mek' , @RangeArcCheck_Mek_glue );
	lua_register( QRL , 'LocatePilot' , @LocatePilot_glue );
	lua_register( QRL , 'GearOperational' , @GearOperational_glue );
	lua_register( QRL , 'GearActive' , @GearActive_glue );
	lua_register( QRL , 'TeamCanSeeTarget' , @TeamCanSeeTarget_glue );
	lua_register( QRL , 'MekCanSeeTarget' , @MekCanSeeTarget_glue );
	lua_register( QRL , 'LocateMekByUID' , @LocateMekByUID_glue );
	lua_register( QRL , 'SeekGear' , @SeekGear_glue );
	lua_register( QRL , 'SeekGearByName' , @SeekGearByName_glue );
	lua_register( QRL , 'SeekGearByPilotName' , @SeekGearByPilotName_glue );
	lua_register( QRL , 'CharCurrentMental' , @CharCurrentMental_glue );
	lua_register( QRL , 'ReactionTime' , @ReactionTime_glue );
	lua_register( QRL , 'IsSexy' , @IsSexy_glue );

	lua_register( QRL , 'TeamOrders' , @TeamOrders_glue );
	lua_register( QRL , 'AmmoPresent' , @AmmoPresent_glue );
	lua_register( QRL , 'AmmoSpent' , @AmmoSpent_glue );
	lua_register( QRL , 'NeedAmmo' , @NeedAmmo_glue );
	lua_register( QRL , 'GoodAmmo' , @GoodAmmo_glue );
	lua_register( QRL , 'InstallAmmo' , @InstallAmmo_glue );
	lua_register( QRL , 'ReloadAmmo' , @ReloadAmmo_glue );
	lua_register( QRL , 'Remaining_of_Ammo' , @Remaining_of_Ammo_glue );

	lua_register( QRL , 'MaxSeparableMode' , @MaxSeparableMode_glue );
	lua_register( QRL , 'IsSeparable' , @IsSeparable_glue );
	lua_register( QRL , 'SeparateWait' , @SeparateWait_glue );

	lua_register( QRL , 'MaxTransformableMode' , @MaxTransformableMode_glue );
	lua_register( QRL , 'Transform_CurrentForm' , @Transform_CurrentForm_glue );
	lua_register( QRL , 'TransformWait' , @TransformWait_glue );
	lua_register( QRL , 'TransformGS' , @TransformGS_glue );

	lua_register( QRL , 'WaitAMinute' , @WaitAMinute_glue );
	lua_register( QRL , 'PrepAction' , @PrepAction_glue );
	lua_register( QRL , 'GearUp' , @GearUp_glue );
	lua_register( QRL , 'RollStep' , @RollStep_glue );
	lua_register( QRL , 'AIRepair' , @AIRepair_glue );
	lua_register( QRL , 'NPC_Flirtation' , @NPC_Flirtation_glue );
	lua_register( QRL , 'NPC_Chatting' , @NPC_Chatting_glue );
	lua_register( QRL , 'AI_Eject' , @AI_Eject_glue );
	lua_register( QRL , 'SafeToFire' , @SafeToFire_glue );
	lua_register( QRL , 'WGoodness' , @WGoodness_glue );
	lua_register( QRL , 'AIAttacker' , @AIAttacker_glue );
	lua_register( QRL , 'AttackerFrontEnd' , @AttackerFrontEnd_glue );
{$IFDEF PATCH_CHEAT}
	lua_register( QRL , 'DoTransformation' , @DoTransformation_glue );
	lua_register( QRL , 'DoPurgeParts' , @DoPurgeParts_glue );
{$ENDIF PATCH_CHEAT}

	lua_register( QRL , 'NPC_CombatTaunt' , @NPC_CombatTaunt_glue );
	lua_register( QRL , 'NPC_Speaks' , @NPC_Speaks_glue );
	lua_register( QRL , 'SoundEffect' , @SoundEffect_glue );

	QRL_attach_glue := True;
end;



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: qrl_glue.pp');
{$ENDIF DEBUG}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: qrl_glue.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
{$ENDIF PATCH_HEART}
