{$IFDEF PATCH_GH}
unit sdltext;
{$MODE FPC}
{
	GearHead: Arena, a roguelike mecha CRPG
	Copyright (C) 2005 Joseph Hewitt

	This library is free software; you can redistribute it and/or modify it
	under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation; either version 2.1 of the License, or (at
	your option) any later version.

	The full text of the LGPL can be found in license.txt.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
	General Public License for more details. 

	You should have received a copy of the GNU Lesser General Public License
	along with this library; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
}

interface

uses
	sysutils, gears_base;

Type
	MoreTextRedrawProcedureType = Procedure;

Procedure MoreText( LList: SAttPtr; FirstLine: Integer; ReDrawer: MoreTextRedrawProcedureType );


implementation

uses
{$IFDEF DEBUG}
	errmsg,
{$ENDIF DEBUG}
{$IFDEF PATCH_I18N}
	i18nmsg,termenc,
{$ENDIF PATCH_I18N}
	gears,ui4gh,
{$IFDEF SDL2}
	SDL2,SDL2_TTF,
{$ELSE SDL2}
	SDL,SDL_TTF,
{$ENDIF SDL2}
	sdlgfx,sdlmenus
{$IFDEF PATCH_HEART}
	,sdlinit
{$ENDIF PATCH_HEART}
	;

Procedure MoreTextRedraw;
var
	MyPromptZone,MyTextZone: TSDL_Rect;
begin
	MyTextZone := ZONE_MoreText.GetRect();
	MyPromptZone := ZONE_MorePrompt.GetRect();

	InfoBox( MyTextZone );
	InfoBox( MyPromptZone );
	CMessage( I18N_MsgString( 'MoreText' , 'Prompt' ) , MyPromptZone , InfoGreen );
end;

Procedure MoreText( LList: SAttPtr; FirstLine: Integer; ReDrawer: MoreTextRedrawProcedureType );
	{ Browse this text file across the majority of the screen. }
	{ Clear the screen upon exiting, though restoration of the }
	{ previous display is someone else's responsibility. }
{$IFDEF PATCH_GH}
var
	MT_tmp: SAttPtr;
{$ENDIF PATCH_GH}

	Procedure DisplayTextHere( const MyZone: TSDL_Rect );
	var
		T: Integer;
		MyDest: TSDL_Rect;
{$IFDEF SDL2}
		MyImage: PSDL_Texture;
		iw, ih: Integer;
{$ELSE SDL2}
		MyImage: PSDL_Surface;
{$ENDIF SDL2}
		CLine: SAttPtr;	{ Current Line }
{$IFDEF PATCH_I18N}
{$ELSE PATCH_I18N}
		PLine: PChar;
{$ENDIF PATCH_I18N}
	begin
		{ Set the clip area. }
{$IFDEF SDL2}
		SDL_RenderSetClipRect( Game_Renderer , @MyZone );
{$ELSE SDL2}
		SDL_SetClipRect( Game_Screen , @MyZone );
{$ENDIF SDL2}
		MyDest := MyZone;
{$IFDEF PATCH_JPSSDL}
		MyDest.X := MyDest.X + Pad_Left;
		MyDest.W := MyDest.W - Pad_Left;
{$ENDIF PATCH_JPSSDL}

		{ Error check. }
{$IFDEF PATCH_GH}
		if ( -1 = FirstLine ) then begin
			FirstLine := NumSAtts( MT_tmp ) - ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) ) + 1;
		end;
		if ( FirstLine < 1 ) then begin
			FirstLine := 1
		end else if ( NumSAtts( MT_tmp ) < FirstLine ) then begin
			FirstLine := NumSAtts( MT_tmp );
		end;
{$ELSE PATCH_GH}
		if FirstLine < 1 then FirstLine := 1
		else if FirstLine > MoreHighFirstLine( LList ) then FirstLine := MoreHighFirstLine( LList );
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
		CLine := RetrieveSAtt( MT_tmp , FirstLine );
{$ELSE PATCH_GH}
		CLine := RetrieveSATt( LList , FirstLine );
{$ENDIF PATCH_GH}
		for t := 1 to ( MyZone.H  div  TTF_FontLineSkip( game_font ) ) do begin
			if CLine <> Nil then begin
{$IFDEF PATCH_I18N}
				MyImage := I18N_TTF_RenderText( game_font , CLine^.Info , NeutralGrey );
  {$IFDEF SDL2}
				SDL_QueryTexture( MyImage , NIL , NIL , @iw , @ih );
  {$ELSE SDL2}
  {$ENDIF SDL2}
				if TERMINAL_bidiRTL then begin
  {$IFDEF SDL2}
					MyDest.X := MyDest.X + MyDest.W - iw;
  {$ELSE SDL2}
					MyDest.X := MyDest.X + MyDest.W - MyImage^.W;
  {$ENDIF SDL2}
				end;
{$ELSE PATCH_I18N}
				pline := QuickPCopy( CLine^.Info );
				MyImage := TTF_RenderText_Solid( game_font , pline , NeutralGrey );
				sysutils.StrDispose( pline );
{$IFDEF Unix}
				SDL_SetColorKey( MyImage , SDL_SRCCOLORKEY , SDL_MapRGB( MyImage^.Format , 0 , 0, 0 ) );
{$ENDIF}
{$ENDIF PATCH_I18N}

{$IFDEF SDL2}
				MyDest.W := iw;
				MyDest.H := ih;
				SDL_RenderCopy( Game_Renderer , MyImage , NIL , @MyDest );
				SDL_DestroyTexture( MyImage );
{$ELSE SDL2}
				SDL_BlitSurface( MyImage , Nil , Game_Screen , @MyDest );
				SDL_FreeSurface( MyImage );
{$ENDIF SDL2}
				MyDest.Y := MyDest.Y + TTF_FontLineSkip( game_font );
				CLine := CLine^.Next;
			end;
		end;

		if (Animation_Phase div 10 mod 2) = 1 then begin
			if FirstLine > 1 then begin
				MyDest.X := MyZone.X + MyZone.W - 16;
				MyDest.Y := MyZone.Y;
				QuickText('+', MyDest, MenuSelect );
			end;
			if CLine <> Nil then begin
				MyDest.X := MyZone.X + MyZone.W - 16;
				MyDest.Y := MyZone.Y + MyZone.H - TTF_FontLineSkip( game_font );
				QuickText('+', MyDest, MenuSelect );
			end;
		end;

		{ Restore the clip area. }
{$IFDEF SDL2}
		SDL_RenderSetClipRect( Game_Renderer , NIL );
{$ELSE SDL2}
		SDL_SetClipRect( Game_Screen , Nil );
{$ENDIF SDL2}
		GHFlip;
	end;
var
	A: Char;
	MyPromptZone,MyTextZone: TSDL_Rect;
{$IFDEF PATCH_GH}
	RPM: RPGMenuPtr;
	t, t_max: Integer;
	CLine: SAttPtr;	{ Current Line }
	msg: String;
	NextWord: String;
	TheLine: String;
	cont: Boolean;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if SDL_Show_MenuScrollbar then begin
		RPM := CreateRPGMenu( NeutralGrey , MenuSelect , ZONE_MoreText );
		CLine := RetrieveSAtt( LList , 1 );
		t_max := NumSAtts( LList );
		for t := 1 to t_max do begin
			msg := CLine^.Info;
			TheLine := '';
			NextWord := '';
			cont := false;
			repeat
				GetNextLine( TheLine , msg , NextWord , ZONE_MoreText.W , game_font );
				if ('' <> TheLine) then begin
					if cont then begin
						AddRPGMenuItem( RPM , #$0 + TheLine , -1 );
					end else begin
						AddRPGMenuItem( RPM , TheLine , -1 );
						cont := true;
					end;
				end;
				TheLine := NextWord;
			until ('' = TheLine);
			CLine := CLine^.Next;
		end;
		if ( -1 = FirstLine ) then begin
			FirstLine := RPM^.NumItem - ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) ) + 1;
		end;
		if ( FirstLine < 1 ) then begin
			FirstLine := 1
		end else if ( RPM^.NumItem < FirstLine ) then begin
			FirstLine := RPM^.NumItem;
		end;
		RPM^.TopItem := FirstLine;
		SetItemByPosition( RPM, FirstLine );
		SelectMenu( RPM , @MoreTextRedraw );
		DisposeRPGMenu( RPM );
	end else begin
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
	MT_tmp := NIL;
	CLine := LList;
	while ( NIL <> CLine ) do begin
		msg := CLine^.Info;
		TheLine := '';
		NextWord := '';
		repeat
			GetNextLine( TheLine , msg , NextWord , ZONE_MoreText.W , game_font );
			if ( '' <> TheLine ) then begin
				StoreSAtt( MT_tmp , TheLine );
			end;
			TheLine := NextWord;
		until ( '' = TheLine );
		CLine := CLine^.Next;
	end;

	{ Display the screen. }
	MyTextZone := ZONE_MoreText.GetRect();
	MyPromptZone := ZONE_MorePrompt.GetRect();
	if Redrawer <> Nil then Redrawer();
	InfoBox( MyTextZone );
	InfoBox( MyPromptZone );
	CMessage( MsgString( 'MORETEXT_Prompt' ) , MyPromptZone , InfoGreen );
	DisplayTextHere( MyTextZone );
{$ENDIF PATCH_GH}

	repeat
		{ Get input from user. }
		A := RPGKey;

{$IFDEF PATCH_GH}
		if (KBD_NONE = A) then begin
		end else if A = KeyMap[ KMC_MenuUp ].KCode then begin
			A := RPK_Up;
		end else if A = KeyMap[ KMC_MenuDown ].KCode then begin
			A := RPK_Down;
		end else if A = KeyMap[ KMC_PageUp ].KCode then begin
			A := RPK_UpRight;
		end else if A = KeyMap[ KMC_PageDown ].KCode then begin
			A := RPK_DownRight;
		end else if A = KeyMap[ KMC_ScrollUp ].KCode then begin
			A := RPK_UpRight;
		end else if A = KeyMap[ KMC_ScrollDown ].KCode then begin
			A := RPK_DownRight;
		end;
{$ENDIF PATCH_GH}

		{ Possibly process this input. }
		if A = RPK_Down then begin
			Inc( FirstLine );
		end else if A = RPK_Up then begin
			Dec( FirstLine );
{$IFDEF PATCH_GH}
		end else if A = RPK_UpRight then begin
			FirstLine := FirstLine - ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) );
			DisplayTextHere( MyTextZone );
		end else if A = RPK_DownRight then begin
			FirstLine := FirstLine + ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) );
			DisplayTextHere( MyTextZone );
{$ENDIF PATCH_GH}
{$IFDEF PATCH_JPSSDL}
		end else if A = RPK_Left then begin
			FirstLine := FirstLine - ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) );
			DisplayTextHere( MyTextZone );
		end else if A = RPK_Right then begin
			FirstLine := FirstLine + ( ZONE_MoreText.H  div  TTF_FontLineSkip( game_font ) );
			DisplayTextHere( MyTextZone );
{$ENDIF PATCH_JPSSDL}
{$IFDEF PATCH_GH}
		end else if A = RPK_RightButton then begin
			A := #8;
{$ENDIF PATCH_GH}
		end else if A = RPK_TimeEvent then begin
			MyTextZone := ZONE_MoreText.GetRect();
			MyPromptZone := ZONE_MorePrompt.GetRect();
			if Redrawer <> Nil then Redrawer();
			InfoBox( MyTextZone );
			InfoBox( MyPromptZone );
			CMessage( MsgString( 'MORETEXT_Prompt' ) , MyPromptZone , InfoGreen );

			{ Display the screen. }
			DisplayTextHere( MyTextZone );
		end;

	until ( A = #27 ) or ( A = 'Q' ) or ( A = #8 );
{$IFDEF PATCH_GH}
	DisposeSAtt( MT_tmp );
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH}
	end;
{$ENDIF PATCH_GH}
end;


initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: sdltext.pp');
{$ENDIF DEBUG}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: sdltext.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
{$ENDIF PATCH_GH}
